@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -37,19 +37,17 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'itemids' =>		[T_ZBX_INT, O_MAND, P_SYS,	DB_ID,	'isset({favobj})'],
+	'itemids' =>		[T_ZBX_INT, O_MAND, P_SYS,	DB_ID,	null],
 	'period' =>			[T_ZBX_INT, O_OPT, null,	null,	null],
 	'stime' =>			[T_ZBX_STR, O_OPT, null,	null,	null],
+	'isNow' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
 	'filter_task' =>	[T_ZBX_STR, O_OPT, null,	IN(FILTER_TASK_SHOW.','.FILTER_TASK_HIDE.','.FILTER_TASK_MARK.','.FILTER_TASK_INVERT_MARK), null],
 	'filter' =>			[T_ZBX_STR, O_OPT, null,	null,	null],
 	'mark_color' =>		[T_ZBX_STR, O_OPT, null,	IN(MARK_COLOR_RED.','.MARK_COLOR_GREEN.','.MARK_COLOR_BLUE), null],
 	'cmbitemlist' =>	[T_ZBX_INT, O_OPT, null,	DB_ID,	null],
 	'plaintext' =>		[T_ZBX_STR, O_OPT, null,	null,	null],
 	'action' =>			[T_ZBX_STR, O_OPT, P_SYS,	IN('"'.HISTORY_GRAPH.'","'.HISTORY_VALUES.'","'.HISTORY_LATEST.'","'.HISTORY_BATCH_GRAPH.'"'), null],
-	'graphtype' =>      [T_ZBX_INT, O_OPT, null,   IN([GRAPH_TYPE_NORMAL, GRAPH_TYPE_STACKED]), null],
-	// ajax
-	'favobj' =>			[T_ZBX_STR, O_OPT, P_ACT,	null,	null],
-	'favid' =>			[T_ZBX_INT, O_OPT, P_ACT,	null,	null],
+	'graphtype' =>		[T_ZBX_INT, O_OPT, null,   IN([GRAPH_TYPE_NORMAL, GRAPH_TYPE_STACKED]), null],
 	// actions
 	'reset' =>			[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null, null],
 	'cancel' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,	null],
@@ -60,20 +58,6 @@ $fields = [
 ];
 check_fields($fields);
 
-/*
- * Ajax
- */
-if (isset($_REQUEST['favobj'])) {
-	if ($_REQUEST['favobj'] == 'timeline') {
-		navigation_bar_calc('web.item.graph', $_REQUEST['favid'], true);
-	}
-
-	// saving fixed/dynamic setting to profile
-	if ($_REQUEST['favobj'] == 'timelinefixedperiod' && isset($_REQUEST['favid'])) {
-		CProfile::update('web.history.timelinefixed', $_REQUEST['favid'], PROFILE_TYPE_INT);
-	}
-}
-
 if ($page['type'] == PAGE_TYPE_JS || $page['type'] == PAGE_TYPE_HTML_BLOCK) {
 	require_once dirname(__FILE__).'/include/page_footer.php';
 	exit;
@@ -112,6 +96,7 @@ $data = [
 	'action' => getRequest('action'),
 	'period' => getRequest('period'),
 	'stime' => getRequest('stime'),
+	'isNow' => getRequest('isNow'),
 	'plaintext' => isset($_REQUEST['plaintext']),
 	'iv_string' => [ITEM_VALUE_TYPE_LOG => 1, ITEM_VALUE_TYPE_TEXT => 1],
 	'iv_numeric' => [ITEM_VALUE_TYPE_FLOAT => 1, ITEM_VALUE_TYPE_UINT64 => 1],
