@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -56,7 +56,6 @@ $fields = [
 	'add' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'update' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'clone' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'update' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'delete' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'cancel' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'form' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
@@ -216,8 +215,7 @@ elseif (hasRequest('add') || hasRequest('update')) {
 }
 // GO
 elseif (hasRequest('action') && str_in_array(getRequest('action'), ['hostprototype.massenable', 'hostprototype.massdisable']) && hasRequest('group_hostid')) {
-	$enable = (getRequest('action') == 'hostprototype.massenable');
-	$status = $enable ? HOST_STATUS_MONITORED : HOST_STATUS_NOT_MONITORED;
+	$status = (getRequest('action') == 'hostprototype.massenable') ? HOST_STATUS_MONITORED : HOST_STATUS_NOT_MONITORED;
 	$update = [];
 
 	DBstart();
@@ -233,12 +231,8 @@ elseif (hasRequest('action') && str_in_array(getRequest('action'), ['hostprototy
 
 	$updated = count($update);
 
-	$messageSuccess = $enable
-		? _n('Host prototype enabled', 'Host prototypes enabled', $updated)
-		: _n('Host prototype disabled', 'Host prototypes disabled', $updated);
-	$messageFailed = $enable
-		? _n('Cannot enable host prototype', 'Cannot enable host prototypes', $updated)
-		: _n('Cannot disable host prototype', 'Cannot disable host prototypes', $updated);
+	$messageSuccess = _n('Host prototype updated', 'Host prototypes updated', $updated);
+	$messageFailed = _n('Cannot update host prototype', 'Cannot update host prototypes', $updated);
 
 	if ($result) {
 		uncheckTableRows($discoveryRule['itemid']);
@@ -317,6 +311,8 @@ if (isset($_REQUEST['form'])) {
 	}
 
 	// host prototype edit form
+	$templateids = [];
+
 	if (getRequest('hostid') && !getRequest('form_refresh')) {
 		$data['host_prototype'] = array_merge($data['host_prototype'], $hostPrototype);
 
@@ -348,11 +344,25 @@ if (isset($_REQUEST['form'])) {
 				if ($parentHostPrototype) {
 					$data['parents'][] = $parentHostPrototype;
 					$hostPrototypeId = $parentHostPrototype['templateid'];
+					$templateids[] = $parentHostPrototype['parentHost']['hostid'];
 				}
 			}
 		}
 	}
 
+	// Select writable templates
+	$templateids = array_merge(zbx_objectValues($data['host_prototype']['templates'], 'templateid'), $templateids);
+	$data['host_prototype']['writable_templates'] = [];
+
+	if ($templateids) {
+		$data['host_prototype']['writable_templates'] = API::Template()->get([
+			'output' => ['templateid'],
+			'templateids' => $templateids,
+			'editable' => true,
+			'preservekeys' => true
+		]);
+	}
+
 	// order linked templates
 	CArrayHelper::sort($data['host_prototype']['templates'], ['name']);
 
@@ -393,15 +403,15 @@ else {
 
 	$data['paging'] = getPagingLine($data['hostPrototypes'], $sortOrder, $url);
 	// fetch templates linked to the prototypes
-	$templateIds = [];
+	$templateids = [];
 	foreach ($data['hostPrototypes'] as $hostPrototype) {
-		$templateIds = array_merge($templateIds, zbx_objectValues($hostPrototype['templates'], 'templateid'));
+		$templateids = array_merge($templateids, zbx_objectValues($hostPrototype['templates'], 'templateid'));
 	}
-	$templateIds = array_unique($templateIds);
+	$templateids = array_keys(array_flip($templateids));
 
 	$linkedTemplates = API::Template()->get([
 		'output' => ['templateid', 'name'],
-		'templateids' => $templateIds,
+		'templateids' => $templateids,
 		'selectParentTemplates' => ['hostid', 'name']
 	]);
 	$data['linkedTemplates'] = zbx_toHash($linkedTemplates, 'templateid');
@@ -426,11 +436,30 @@ else {
 				];
 				$sourceDiscoveryRuleId = get_realrule_by_itemid_and_hostid($discoveryRule['itemid'], $sourceTemplate['hostid']);
 				$hostPrototype['sourceDiscoveryRuleId'] = $sourceDiscoveryRuleId;
+				$templateids[] = $sourceTemplate['parent_hostid'];
 			}
 		}
 		unset($hostPrototype);
 	}
 
+	foreach ($data['linkedTemplates'] as $linked_template) {
+		foreach ($linked_template['parentTemplates'] as $parent_template) {
+			$templateids[] = $parent_template['templateid'];
+		}
+	}
+
+	// Select writable template IDs.
+	$data['writable_templates'] = [];
+
+	if ($templateids) {
+		$data['writable_templates'] = API::Template()->get([
+			'output' => ['templateid'],
+			'templateids' => $templateids,
+			'editable' => true,
+			'preservekeys' => true
+		]);
+	}
+
 	// render view
 	$itemView = new CView('configuration.host.prototype.list', $data);
 	$itemView->render();
