@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,15 +33,14 @@ require_once dirname(__FILE__).'/include/page_header.php';
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
 	'groupid'			=> [T_ZBX_INT, O_OPT, P_SYS,	DB_ID,				null],
-	'new_httpstep'		=> [T_ZBX_STR, O_OPT, null,	null,				null],
-	'sel_step'			=> [T_ZBX_INT, O_OPT, null,	BETWEEN(0, 65534),	null],
+	'new_httpstep'		=> [T_ZBX_STR, O_OPT, P_NO_TRIM,	null,				null],
 	'group_httptestid'	=> [T_ZBX_INT, O_OPT, null,	DB_ID,				null],
 	// form
 	'hostid'          => [T_ZBX_INT, O_OPT, P_SYS, DB_ID.NOT_ZERO,          'isset({form}) || isset({add}) || isset({update})'],
 	'applicationid'   => [T_ZBX_INT, O_OPT, null,  DB_ID,                   null, _('Application')],
 	'httptestid'      => [T_ZBX_INT, O_NO,  P_SYS, DB_ID,                   'isset({form}) && {form} == "update"'],
 	'name'            => [T_ZBX_STR, O_OPT, null,  NOT_EMPTY,               'isset({add}) || isset({update})', _('Name')],
-	'delay'           => [T_ZBX_INT, O_OPT, null,  BETWEEN(1, SEC_PER_DAY), 'isset({add}) || isset({update})', _('Update interval (in sec)')],
+	'delay'           => [T_ZBX_STR, O_OPT, null,  null,					'isset({add}) || isset({update})'],
 	'retries'         => [T_ZBX_INT, O_OPT, null,  BETWEEN(1, 10),          'isset({add}) || isset({update})',
 		_('Attempts')
 	],
@@ -50,8 +49,8 @@ $fields = [
 	'agent_other'     => [T_ZBX_STR, O_OPT, null, null,
 		'(isset({add}) || isset({update})) && {agent} == '.ZBX_AGENT_OTHER
 	],
-	'variables'       => [T_ZBX_STR, O_OPT, null,  null,                    'isset({add}) || isset({update})'],
-	'steps'           => [T_ZBX_STR, O_OPT, null,  null,                    'isset({add}) || isset({update})', _('Steps')],
+	'pairs'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    null],
+	'steps'           => [T_ZBX_STR, O_OPT, P_NO_TRIM,  null,                    'isset({add}) || isset({update})', _('Steps')],
 	'authentication'  => [T_ZBX_INT, O_OPT, null,  IN('0,1,2'),             'isset({add}) || isset({update})'],
 	'http_user'       => [T_ZBX_STR, O_OPT, null,  NOT_EMPTY,               '(isset({add}) || isset({update})) && isset({authentication}) && ({authentication} == '.HTTPTEST_AUTH_BASIC.
 		' || {authentication} == '.HTTPTEST_AUTH_NTLM.')', _('User')],
@@ -63,7 +62,6 @@ $fields = [
 	'templated'			=> [T_ZBX_STR, O_OPT, null,	null,				null],
 	'verify_host'		=> [T_ZBX_STR, O_OPT, null,	null,				null],
 	'verify_peer'		=> [T_ZBX_STR, O_OPT, null,	null,				null],
-	'headers'			=> [T_ZBX_STR, O_OPT, null, null,					'isset({add}) || isset({update})'],
 	'ssl_cert_file'		=> [T_ZBX_STR, O_OPT, null, null,					'isset({add}) || isset({update})'],
 	'ssl_key_file'		=> [T_ZBX_STR, O_OPT, null, null,					'isset({add}) || isset({update})'],
 	'ssl_key_password'	=> [T_ZBX_STR, O_OPT, P_NO_TRIM, null,				'isset({add}) || isset({update})'],
@@ -111,12 +109,21 @@ if (isset($_REQUEST['httptestid']) || !empty($_REQUEST['group_httptestid'])) {
 	if (!empty($_REQUEST['group_httptestid'])) {
 		$testIds = array_merge($testIds, $_REQUEST['group_httptestid']);
 	}
-	if (!API::HttpTest()->isWritable($testIds)) {
-		access_deny();
+	if ($testIds) {
+		$testIds = array_unique($testIds);
+
+		$count = API::HttpTest()->get([
+			'countOutput' => true,
+			'httptestids' => $testIds,
+			'editable' => true
+		]);
+
+		if ($count != count($testIds)) {
+			access_deny();
+		}
 	}
 }
-$hostId = getRequest('hostid');
-if ($hostId && !API::Host()->isWritable([$hostId])) {
+if (getRequest('hostid') && !isWritableHostTemplates([getRequest('hostid')])) {
 	access_deny();
 }
 
@@ -192,12 +199,10 @@ elseif (hasRequest('del_history') && hasRequest('httptestid')) {
 }
 elseif (hasRequest('add') || hasRequest('update')) {
 	if (hasRequest('update')) {
-		$action = AUDIT_ACTION_UPDATE;
 		$messageTrue = _('Web scenario updated');
 		$messageFalse = _('Cannot update web scenario');
 	}
 	else {
-		$action = AUDIT_ACTION_ADD;
 		$messageTrue = _('Web scenario added');
 		$messageFalse = _('Cannot add web scenario');
 	}
@@ -212,40 +217,89 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		}
 
 		$steps = getRequest('steps', []);
-		if (!empty($steps)) {
-			$i = 1;
-			foreach ($steps as $stepNumber => &$step) {
-				$step['no'] = $i++;
-				$step['follow_redirects'] = $step['follow_redirects']
-					? HTTPTEST_STEP_FOLLOW_REDIRECTS_ON
-					: HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF;
-				$step['retrieve_mode'] = $step['retrieve_mode']
-					? HTTPTEST_STEP_RETRIEVE_MODE_HEADERS
-					: HTTPTEST_STEP_RETRIEVE_MODE_CONTENT;
+		$field_names = ['headers', 'variables', 'post_fields', 'query_fields'];
+		$i = 1;
+
+		foreach ($steps as &$step) {
+			$step['no'] = $i++;
+			$step['follow_redirects'] = $step['follow_redirects']
+				? HTTPTEST_STEP_FOLLOW_REDIRECTS_ON
+				: HTTPTEST_STEP_FOLLOW_REDIRECTS_OFF;
+			$step['retrieve_mode'] = $step['retrieve_mode']
+				? HTTPTEST_STEP_RETRIEVE_MODE_HEADERS
+				: HTTPTEST_STEP_RETRIEVE_MODE_CONTENT;
+
+			foreach ($field_names as $field_name) {
+				$step[$field_name] = [];
 			}
-			unset($step);
+
+			if (array_key_exists('pairs', $step)) {
+				foreach ($field_names as $field_name) {
+					foreach ($step['pairs'] as $pair) {
+						if (array_key_exists('type', $pair) && $field_name === $pair['type'] &&
+							((array_key_exists('name', $pair) && $pair['name'] !== '') ||
+							(array_key_exists('value', $pair) && $pair['value'] !== ''))) {
+							$step[$field_name][] = [
+								'name' => (array_key_exists('name', $pair) ? $pair['name'] : ''),
+								'value' => (array_key_exists('value', $pair) ? $pair['value'] : '')
+							];
+						}
+					}
+				}
+				unset($step['pairs']);
+			}
+
+			foreach ($step['variables'] as &$variable) {
+				$variable['name'] = trim($variable['name']);
+			}
+			unset($variable);
+
+			if ($step['post_type'] == ZBX_POSTTYPE_FORM) {
+				$step['posts'] = $step['post_fields'];
+			}
+			unset($step['post_fields'], $step['post_type']);
 		}
+		unset($step);
 
 		$httpTest = [
 			'hostid' => $_REQUEST['hostid'],
 			'name' => $_REQUEST['name'],
 			'authentication' => $_REQUEST['authentication'],
-			'applicationid' => getRequest('applicationid'),
-			'delay' => $_REQUEST['delay'],
+			'applicationid' => getRequest('applicationid', 0),
+			'delay' => getRequest('delay', DB::getDefault('httptest', 'delay')),
 			'retries' => $_REQUEST['retries'],
-			'status' => isset($_REQUEST['status']) ? 0 : 1,
+			'status' => hasRequest('status') ? HTTPTEST_STATUS_ACTIVE : HTTPTEST_STATUS_DISABLED,
 			'agent' => hasRequest('agent_other') ? getRequest('agent_other') : getRequest('agent'),
-			'variables' => $_REQUEST['variables'],
+			'variables' => [],
 			'http_proxy' => $_REQUEST['http_proxy'],
 			'steps' => $steps,
+			'http_user' => ($_REQUEST['authentication'] == HTTPTEST_AUTH_NONE) ? '' : $_REQUEST['http_user'],
+			'http_password' => ($_REQUEST['authentication'] == HTTPTEST_AUTH_NONE) ? '' : $_REQUEST['http_password'],
 			'verify_peer' => getRequest('verify_peer', HTTPTEST_VERIFY_PEER_OFF),
 			'verify_host' => getRequest('verify_host', HTTPTEST_VERIFY_HOST_OFF),
 			'ssl_cert_file' => getRequest('ssl_cert_file'),
 			'ssl_key_file' => getRequest('ssl_key_file'),
 			'ssl_key_password' => getRequest('ssl_key_password'),
-			'headers' => getRequest('headers')
+			'headers' => []
 		];
 
+		foreach (getRequest('pairs', []) as $pair) {
+			if (array_key_exists('type', $pair) && in_array($pair['type'], ['variables', 'headers']) &&
+				((array_key_exists('name', $pair) && $pair['name'] !== '') ||
+				(array_key_exists('value', $pair) && $pair['value'] !== ''))) {
+
+				$httpTest[$pair['type']][] = [
+					'name' => (array_key_exists('name', $pair) ? $pair['name'] : ''),
+					'value' => (array_key_exists('value', $pair) ? $pair['value'] : '')
+				];
+			}
+		}
+
+		foreach ($httpTest['variables'] as &$variable) {
+			$variable['name'] = trim($variable['name']);
+		}
+		unset($variable);
+
 		if ($new_application) {
 			$exApp = API::Application()->get([
 				'output' => ['applicationid', 'flags'],
@@ -279,15 +333,6 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			}
 		}
 
-		if ($_REQUEST['authentication'] != HTTPTEST_AUTH_NONE) {
-			$httpTest['http_user'] = $_REQUEST['http_user'];
-			$httpTest['http_password'] = $_REQUEST['http_password'];
-		}
-		else {
-			$httpTest['http_user'] = '';
-			$httpTest['http_password'] = '';
-		}
-
 		if (isset($_REQUEST['httptestid'])) {
 			// unset fields that did not change
 			$dbHttpTest = API::HttpTest()->get([
@@ -299,10 +344,50 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			$dbHttpSteps = zbx_toHash($dbHttpTest['steps'], 'httpstepid');
 
 			$httpTest = CArrayHelper::unsetEqualValues($httpTest, $dbHttpTest, ['applicationid']);
+			foreach (['headers', 'variables'] as $field_name) {
+				if (count($httpTest[$field_name]) !== count($dbHttpTest[$field_name])) {
+					continue;
+				}
+
+				$changed = false;
+				foreach ($httpTest[$field_name] as $key => $field) {
+					if ($dbHttpTest[$field_name][$key]['name'] !== $field['name']
+							|| $dbHttpTest[$field_name][$key]['value'] !== $field['value']) {
+						$changed = true;
+						break;
+					}
+				}
+
+				if (!$changed) {
+					unset($httpTest[$field_name]);
+				}
+			}
+
 			foreach ($httpTest['steps'] as $snum => $step) {
-				if (isset($step['httpstepid']) && isset($dbHttpSteps[$step['httpstepid']])) {
-					$newStep = CArrayHelper::unsetEqualValues($step, $dbHttpSteps[$step['httpstepid']], ['httpstepid']);
-					$httpTest['steps'][$snum] = $newStep;
+				if (array_key_exists('httpstepid', $step) && array_key_exists($step['httpstepid'], $dbHttpSteps)) {
+					$db_step = $dbHttpSteps[$step['httpstepid']];
+					$new_step = CArrayHelper::unsetEqualValues($step, $db_step, ['httpstepid']);
+					foreach (['headers', 'variables', 'posts', 'query_fields'] as $field_name) {
+						if (!array_key_exists($field_name, $new_step)
+								|| !is_array($new_step[$field_name]) || !is_array($db_step[$field_name])
+								|| count($new_step[$field_name]) !== count($db_step[$field_name])) {
+							continue;
+						}
+
+						$changed = false;
+						foreach ($new_step[$field_name] as $key => $field) {
+							if ($db_step[$field_name][$key]['name'] !== $field['name']
+									|| $db_step[$field_name][$key]['value'] !== $field['value']) {
+								$changed = true;
+								break;
+							}
+						}
+
+						if (!$changed) {
+							unset($new_step[$field_name]);
+						}
+					}
+					$httpTest['steps'][$snum] = $new_step;
 				}
 			}
 
@@ -316,6 +401,11 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			}
 		}
 		else {
+			foreach ($httpTest['steps'] as &$step) {
+				unset($step['httptestid'], $step['httpstepid']);
+			}
+			unset($step);
+
 			$result = API::HttpTest()->create($httpTest);
 			if (!$result) {
 				throw new Exception();
@@ -326,11 +416,6 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			$httpTestId = reset($result['httptestids']);
 		}
 
-		$host = get_host_by_hostid($_REQUEST['hostid']);
-		add_audit($action, AUDIT_RESOURCE_SCENARIO,
-			_('Web scenario').' ['.getRequest('name').'] ['.$httpTestId.'] '._('Host').' ['.$host['name'].']'
-		);
-
 		unset($_REQUEST['form']);
 		show_messages(true, $messageTrue);
 		DBend(true);
@@ -347,56 +432,26 @@ elseif (hasRequest('add') || hasRequest('update')) {
 }
 elseif (hasRequest('action') && str_in_array(getRequest('action'), ['httptest.massenable', 'httptest.massdisable'])
 		&& hasRequest('group_httptestid') && is_array(getRequest('group_httptestid'))) {
-	$result = true;
-	$httpTestIds = getRequest('group_httptestid');
 	$enable = (getRequest('action') === 'httptest.massenable');
 	$status = $enable ? HTTPTEST_STATUS_ACTIVE : HTTPTEST_STATUS_DISABLED;
-	$statusName = $enable ? 'enabled' : 'disabled';
-	$auditAction = $enable ? AUDIT_ACTION_ENABLE : AUDIT_ACTION_DISABLE;
 	$updated = 0;
+	$result = true;
 
-	$httpTests = API::HttpTest()->get([
-		'output' => ['httptestid', 'name', 'status'],
-		'selectHosts' => ['name'],
-		'httptestids' => $httpTestIds,
-		'editable' => true
-	]);
+	$upd_httptests = [];
 
-	if ($httpTests) {
-		$httpTestsToUpdate = [];
-
-		DBstart();
-
-		foreach ($httpTests as $httpTest) {
-			// change status if it's necessary
-			if ($httpTest['status'] != $status) {
-				$httpTestsToUpdate[] = [
-					'httptestid' => $httpTest['httptestid'],
-					'status' => $status
-				];
-			}
-		}
-
-		if ($httpTestsToUpdate) {
-			$result = API::HttpTest()->update($httpTestsToUpdate);
-
-			if ($result) {
-				foreach ($httpTests as $httpTest) {
-					$host = reset($httpTest['hosts']);
-
-					add_audit($auditAction, AUDIT_RESOURCE_SCENARIO,
-						_('Web scenario').' ['.$httpTest['name'].'] ['.$httpTest['httptestid'].'] '.
-							_('Host').' ['.$host['name'].'] '.$statusName
-					);
-				}
-			}
-		}
-
-		$result = DBend($result);
-
-		$updated = count($httpTests);
+	foreach (getRequest('group_httptestid') as $httptestid) {
+		$upd_httptests[] = [
+			'httptestid' => $httptestid,
+			'status' => $status
+		];
 	}
 
+	if ($upd_httptests) {
+		$result = (bool) API::HttpTest()->update($upd_httptests);
+	}
+
+	$updated = count($upd_httptests);
+
 	$messageSuccess = $enable
 		? _n('Web scenario enabled', 'Web scenarios enabled', $updated)
 		: _n('Web scenario disabled', 'Web scenarios disabled', $updated);
@@ -508,44 +563,108 @@ if (isset($_REQUEST['form'])) {
 		array_shift($data['templates']);
 	}
 
-	if ((isset($_REQUEST['httptestid']) && !isset($_REQUEST['form_refresh']))) {
-		$dbHttpTest = DBfetch(DBselect(
-			'SELECT ht.*'.
-			' FROM httptest ht'.
-			' WHERE ht.httptestid='.zbx_dbstr($_REQUEST['httptestid'])
-		));
+	if (hasRequest('httptestid') && !hasRequest('form_refresh')) {
+		$db_httptests = API::HttpTest()->get([
+			'output' => ['name', 'applicationid', 'delay', 'retries', 'status', 'agent', 'authentication',
+				'http_user', 'http_password', 'http_proxy', 'templateid', 'verify_peer', 'verify_host', 'ssl_cert_file',
+				'ssl_key_file', 'ssl_key_password', 'headers', 'variables'
+			],
+			'selectSteps' => ['httpstepid', 'name', 'no', 'url', 'timeout', 'posts', 'required', 'status_codes',
+				'follow_redirects', 'retrieve_mode', 'headers', 'variables', 'query_fields', 'post_type'
+			],
+			'httptestids' => getRequest('httptestid')
+		]);
 
-		$data['name'] = $dbHttpTest['name'];
-		$data['applicationid'] = $dbHttpTest['applicationid'];
+		$db_httptest = $db_httptests[0];
+
+		$data['name'] = $db_httptest['name'];
+		$data['applicationid'] = $db_httptest['applicationid'];
 		$data['new_application'] = '';
-		$data['delay'] = $dbHttpTest['delay'];
-		$data['retries'] = $dbHttpTest['retries'];
-		$data['status'] = $dbHttpTest['status'];
+		$data['delay'] = $db_httptest['delay'];
+		$data['retries'] = $db_httptest['retries'];
+		$data['status'] = $db_httptest['status'];
 
 		$data['agent'] = ZBX_AGENT_OTHER;
-		$data['agent_other'] = $dbHttpTest['agent'];
+		$data['agent_other'] = $db_httptest['agent'];
 
 		foreach (userAgents() as $userAgents) {
-			if (array_key_exists($dbHttpTest['agent'], $userAgents)) {
-				$data['agent'] = $dbHttpTest['agent'];
+			if (array_key_exists($db_httptest['agent'], $userAgents)) {
+				$data['agent'] = $db_httptest['agent'];
 				$data['agent_other'] = '';
 				break;
 			}
 		}
 
-		$data['variables'] = $dbHttpTest['variables'];
-		$data['authentication'] = $dbHttpTest['authentication'];
-		$data['http_user'] = $dbHttpTest['http_user'];
-		$data['http_password'] = $dbHttpTest['http_password'];
-		$data['http_proxy'] = $dbHttpTest['http_proxy'];
-		$data['templated'] = (bool) $dbHttpTest['templateid'];
-		$data['headers'] = $dbHttpTest['headers'];
-		$data['verify_peer'] = $dbHttpTest['verify_peer'];
-		$data['verify_host'] = $dbHttpTest['verify_host'];
-		$data['ssl_cert_file'] = $dbHttpTest['ssl_cert_file'];
-		$data['ssl_key_file'] = $dbHttpTest['ssl_key_file'];
-		$data['ssl_key_password'] = $dbHttpTest['ssl_key_password'];
-		$data['steps'] = DBfetchArray(DBselect('SELECT h.* FROM httpstep h WHERE h.httptestid='.zbx_dbstr($_REQUEST['httptestid']).' ORDER BY h.no'));
+		$id = 1;
+		$data['pairs'] = [];
+
+		$fields = [
+			'headers' => 'headers',
+			'variables' => 'variables'
+		];
+
+		CArrayHelper::sort($db_httptest['variables'], ['name']);
+
+		foreach ($fields as $type => $field_name) {
+			foreach ($db_httptest[$field_name] as $pair) {
+				$data['pairs'][] = [
+					'id' => $id++,
+					'type' => $type,
+					'name' => $pair['name'],
+					'value' => $pair['value']
+				];
+			}
+		}
+
+		$data['authentication'] = $db_httptest['authentication'];
+		$data['http_user'] = $db_httptest['http_user'];
+		$data['http_password'] = $db_httptest['http_password'];
+		$data['http_proxy'] = $db_httptest['http_proxy'];
+		$data['templated'] = (bool) $db_httptest['templateid'];
+
+		$data['verify_peer'] = $db_httptest['verify_peer'];
+		$data['verify_host'] = $db_httptest['verify_host'];
+		$data['ssl_cert_file'] = $db_httptest['ssl_cert_file'];
+		$data['ssl_key_file'] = $db_httptest['ssl_key_file'];
+		$data['ssl_key_password'] = $db_httptest['ssl_key_password'];
+		$data['steps'] = $db_httptest['steps'];
+		CArrayHelper::sort($data['steps'], ['no']);
+
+		$fields = [
+			'headers' => 'headers',
+			'variables' => 'variables',
+			'query_fields' => 'query_fields',
+			'post_fields' => 'posts'
+		];
+
+		foreach ($data['steps'] as &$step) {
+			$id = 0;
+			$step['pairs'] = [];
+
+			CArrayHelper::sort($step['variables'], ['name']);
+
+			foreach ($fields as $type => $field_name) {
+				if ($field_name !== 'posts' || $step['post_type'] == ZBX_POSTTYPE_FORM) {
+					foreach ($step[$field_name] as $pair) {
+						$step['pairs'][] = [
+							'id' => $id++,
+							'type' => $type,
+							'name' => $pair['name'],
+							'value' => $pair['value']
+						];
+					}
+
+					if ($field_name === 'posts') {
+						$step['posts'] = '';
+					}
+					else {
+						unset($step[$field_name]);
+					}
+				}
+			}
+		}
+		unset($step);
+
 	}
 	else {
 		if (isset($_REQUEST['form_refresh'])) {
@@ -558,7 +677,7 @@ if (isset($_REQUEST['form'])) {
 		$data['name'] = getRequest('name', '');
 		$data['applicationid'] = getRequest('applicationid');
 		$data['new_application'] = getRequest('new_application', '');
-		$data['delay'] = getRequest('delay', 60);
+		$data['delay'] = getRequest('delay', DB::getDefault('httptest', 'delay'));
 		$data['retries'] = getRequest('retries', 1);
 
 		$data['agent'] = getRequest('agent', ZBX_DEFAULT_AGENT);
@@ -574,19 +693,18 @@ if (isset($_REQUEST['form'])) {
 			}
 		}
 
-		$data['variables'] = getRequest('variables', []);
 		$data['authentication'] = getRequest('authentication', HTTPTEST_AUTH_NONE);
 		$data['http_user'] = getRequest('http_user', '');
 		$data['http_password'] = getRequest('http_password', '');
 		$data['http_proxy'] = getRequest('http_proxy', '');
 		$data['templated'] = (bool) getRequest('templated');
 		$data['steps'] = getRequest('steps', []);
-		$data['headers'] = getRequest('headers');
 		$data['verify_peer'] = getRequest('verify_peer');
 		$data['verify_host'] = getRequest('verify_host');
 		$data['ssl_cert_file'] = getRequest('ssl_cert_file');
 		$data['ssl_key_file'] = getRequest('ssl_key_file');
 		$data['ssl_key_password'] = getRequest('ssl_key_password');
+		$data['pairs'] = array_values(getRequest('pairs', []));
 	}
 
 	$data['application_list'] = [];
