@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@
  */
 class CHttpTestManager {
 
-	const ITEM_HISTORY = 30;
-	const ITEM_TRENDS = 90;
+	const ITEM_HISTORY = '30d';
+	const ITEM_TRENDS = '90d';
 
 	/**
 	 * Changed steps names.
@@ -105,24 +105,15 @@ class CHttpTestManager {
 	public function create(array $httpTests) {
 		$httpTestIds = DB::insert('httptest', $httpTests);
 
-		foreach ($httpTests as $hnum => $httpTest) {
-			$httpTests[$hnum]['httptestid'] = $httpTestIds[$hnum];
-
+		foreach ($httpTests as $hnum => &$httpTest) {
 			$httpTest['httptestid'] = $httpTestIds[$hnum];
+
 			$this->createHttpTestItems($httpTest);
 			$this->createStepsReal($httpTest, $httpTest['steps']);
 		}
+		unset($httpTest);
 
-		// TODO: REMOVE info
-		$dbCursor = DBselect(
-			'SELECT ht.name,h.name AS hostname'.
-			' FROM httptest ht'.
-				' INNER JOIN hosts h ON ht.hostid=h.hostid'.
-			' WHERE '.dbConditionInt('ht.httptestid', zbx_objectValues($httpTests, 'httptestid'))
-		);
-		while ($httpTest = DBfetch($dbCursor)) {
-			info(_s('Created: Web scenario "%1$s" on "%2$s".', $httpTest['name'], $httpTest['hostname']));
-		}
+		$this->updateHttpTestFields($httpTests, 'create');
 
 		return $httpTests;
 	}
@@ -130,16 +121,22 @@ class CHttpTestManager {
 	/**
 	 * Update http tests.
 	 *
-	 * @param array $httpTests
+	 * @param array $httptests
 	 *
 	 * @return array
 	 */
-	public function update(array $httpTests) {
-		$httpTestIds = zbx_objectValues($httpTests, 'httptestid');
-		$dbHttpTest = API::HttpTest()->get([
-			'output' => API_OUTPUT_EXTEND,
-			'httptestids' => $httpTestIds,
-			'selectSteps' => API_OUTPUT_EXTEND,
+	public function update(array $httptests) {
+		$httptestids = zbx_objectValues($httptests, 'httptestid');
+
+		$db_httptests = API::HttpTest()->get([
+			'output' => ['httptestid', 'name', 'applicationid', 'delay', 'status', 'agent', 'authentication',
+				'http_user', 'http_password', 'hostid', 'templateid', 'http_proxy', 'retries', 'ssl_cert_file',
+				'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host'
+			],
+			'httptestids' => $httptestids,
+			'selectSteps' => ['httpstepid', 'name', 'no', 'url', 'timeout', 'posts', 'required', 'status_codes',
+				'follow_redirects', 'retrieve_mode'
+			],
 			'editable' => true,
 			'preservekeys' => true
 		]);
@@ -148,10 +145,16 @@ class CHttpTestManager {
 		$steps_create = [];
 		$steps_update = [];
 
-		foreach ($httpTests as $key => $httpTest) {
+		foreach ($httptests as $key => $httptest) {
+			$db_httptest = $db_httptests[$httptest['httptestid']];
+
+			if (array_key_exists('delay', $httptest) && $db_httptest['delay'] != $httptest['delay']) {
+				$httptest['nextcheck'] = 0;
+			}
+
 			DB::update('httptest', [
-				'values' => $httpTest,
-				'where' => ['httptestid' => $httpTest['httptestid']]
+				'values' => $httptest,
+				'where' => ['httptestid' => $httptest['httptestid']]
 			]);
 
 			$checkItemsUpdate = [];
@@ -160,21 +163,21 @@ class CHttpTestManager {
 			$dbCheckItems = DBselect(
 				'SELECT i.itemid,hi.type'.
 				' FROM items i,httptestitem hi'.
-				' WHERE hi.httptestid='.zbx_dbstr($httpTest['httptestid']).
+				' WHERE hi.httptestid='.zbx_dbstr($httptest['httptestid']).
 					' AND hi.itemid=i.itemid'
 			);
 			while ($checkitem = DBfetch($dbCheckItems)) {
 				$itemids[] = $checkitem['itemid'];
 
-				if (isset($httpTest['name'])) {
-					$updateFields['key_'] = $this->getTestKey($checkitem['type'], $httpTest['name']);
+				if (isset($httptest['name'])) {
+					$updateFields['key_'] = $this->getTestKey($checkitem['type'], $httptest['name']);
 				}
 
-				if (isset($httpTest['status'])) {
-					$updateFields['status'] = (HTTPTEST_STATUS_ACTIVE == $httpTest['status']) ? ITEM_STATUS_ACTIVE : ITEM_STATUS_DISABLED;
+				if (isset($httptest['status'])) {
+					$updateFields['status'] = (HTTPTEST_STATUS_ACTIVE == $httptest['status']) ? ITEM_STATUS_ACTIVE : ITEM_STATUS_DISABLED;
 				}
-				if (isset($httpTest['delay'])) {
-					$updateFields['delay'] = $httpTest['delay'];
+				if (isset($httptest['delay'])) {
+					$updateFields['delay'] = $httptest['delay'];
 				}
 				if (!empty($updateFields)) {
 					$checkItemsUpdate[] = [
@@ -185,15 +188,14 @@ class CHttpTestManager {
 			}
 			DB::update('items', $checkItemsUpdate);
 
-			if (isset($httpTest['applicationid'])) {
-				$this->updateItemsApplications($itemids, $httpTest['applicationid']);
+			if (isset($httptest['applicationid'])) {
+				$this->updateItemsApplications($itemids, $httptest['applicationid']);
 			}
 
-			if (array_key_exists('steps', $httpTest)) {
-				$db_http_test = $dbHttpTest[$httpTest['httptestid']];
-				$dbSteps = zbx_toHash($db_http_test['steps'], 'httpstepid');
+			if (array_key_exists('steps', $httptest)) {
+				$dbSteps = zbx_toHash($db_httptest['steps'], 'httpstepid');
 
-				foreach ($httpTest['steps'] as $webstep) {
+				foreach ($httptest['steps'] as $webstep) {
 					if (isset($webstep['httpstepid']) && isset($dbSteps[$webstep['httpstepid']])) {
 						$steps_update[$key][] = $webstep;
 						unset($dbSteps[$webstep['httpstepid']]);
@@ -201,10 +203,6 @@ class CHttpTestManager {
 					elseif (!isset($webstep['httpstepid'])) {
 						$steps_create[$key][] = $webstep;
 					}
-
-					if ($db_http_test['templateid'] != 0) {
-						unset($dbSteps[$webstep['httpstepid']]);
-					}
 				}
 
 				$stepidsDelete = array_keys($dbSteps);
@@ -222,11 +220,11 @@ class CHttpTestManager {
 				}
 
 				// IF application ID was not set, use the ID from DB so new items can be linked.
-				if (!array_key_exists('applicationid', $httpTest)) {
-					$httpTest['applicationid'] = $db_http_test['applicationid'];
+				if (!array_key_exists('applicationid', $httptest)) {
+					$httptest['applicationid'] = $db_httptest['applicationid'];
 				}
-				elseif (bccomp($httpTest['applicationid'], $db_http_test['applicationid'])) {
-					unset($httpTest['applicationid']);
+				elseif (bccomp($httptest['applicationid'], $db_httptest['applicationid'])) {
+					unset($httptest['applicationid']);
 				}
 			}
 		}
@@ -236,36 +234,36 @@ class CHttpTestManager {
 			API::Item()->delete($deleteStepItemIds, true);
 		}
 
-		foreach ($httpTests as $key => $httpTest) {
-			if (array_key_exists('steps', $httpTest)) {
+		foreach ($httptests as $key => $httptest) {
+			if (array_key_exists('steps', $httptest)) {
 				if (array_key_exists($key, $steps_update)) {
-					$this->updateStepsReal($httpTest, $steps_update[$key]);
+					$this->updateStepsReal($httptest, $steps_update[$key]);
 				}
 
 				if (array_key_exists($key, $steps_create)) {
-					$this->createStepsReal($httpTest, $steps_create[$key]);
+					$this->createStepsReal($httptest, $steps_create[$key]);
 				}
 			}
 			else {
-				if (isset($httpTest['applicationid'])) {
+				if (isset($httptest['applicationid'])) {
 					$dbStepIds = DBfetchColumn(DBselect(
 						'SELECT i.itemid'.
 						' FROM items i'.
 							' INNER JOIN httpstepitem hi ON hi.itemid=i.itemid'.
-						' WHERE '.dbConditionInt('hi.httpstepid', zbx_objectValues($dbHttpTest[$httpTest['httptestid']]['steps'], 'httpstepid')))
+						' WHERE '.dbConditionInt('hi.httpstepid', zbx_objectValues($db_httptests[$httptest['httptestid']]['steps'], 'httpstepid')))
 						, 'itemid'
 					);
-					$this->updateItemsApplications($dbStepIds, $httpTest['applicationid']);
+					$this->updateItemsApplications($dbStepIds, $httptest['applicationid']);
 				}
 
-				if (isset($httpTest['status'])) {
-					$status = ($httpTest['status'] == HTTPTEST_STATUS_ACTIVE) ? ITEM_STATUS_ACTIVE : ITEM_STATUS_DISABLED;
+				if (isset($httptest['status'])) {
+					$status = ($httptest['status'] == HTTPTEST_STATUS_ACTIVE) ? ITEM_STATUS_ACTIVE : ITEM_STATUS_DISABLED;
 
 					$itemIds = DBfetchColumn(DBselect(
 						'SELECT hsi.itemid'.
 							' FROM httpstep hs,httpstepitem hsi'.
 							' WHERE hs.httpstepid=hsi.httpstepid'.
-								' AND hs.httptestid='.zbx_dbstr($httpTest['httptestid'])
+								' AND hs.httptestid='.zbx_dbstr($httptest['httptestid'])
 					), 'itemid');
 
 					DB::update('items', [
@@ -276,18 +274,9 @@ class CHttpTestManager {
 			}
 		}
 
-		// TODO: REMOVE info
-		$dbCursor = DBselect(
-			'SELECT ht.name,h.name AS hostname'.
-			' FROM httptest ht'.
-				' INNER JOIN hosts h ON ht.hostid=h.hostid'.
-			' WHERE '.dbConditionInt('ht.httptestid', zbx_objectValues($httpTests, 'httptestid'))
-		);
-		while ($httpTest = DBfetch($dbCursor)) {
-			info(_s('Updated: Web scenario "%1$s" on "%2$s".', $httpTest['name'], $httpTest['hostname']));
-		}
+		$this->updateHttpTestFields($httptests, 'update');
 
-		return $httpTests;
+		return $httptests;
 	}
 
 	/**
@@ -299,27 +288,17 @@ class CHttpTestManager {
 	public function link($templateId, $hostIds) {
 		$hostIds = zbx_toArray($hostIds);
 
-		$httpTests = [];
-		$dbCursor = DBselect(
-			'SELECT ht.httptestid,ht.name,ht.applicationid,ht.delay,ht.status,ht.variables,ht.agent,'.
-				'ht.authentication,ht.http_user,ht.http_password,ht.http_proxy,ht.retries,ht.hostid,ht.templateid,'.
-				'ht.headers,ht.verify_peer,ht.verify_host,ht.ssl_cert_file,ht.ssl_key_file,ht.ssl_key_password'.
-			' FROM httptest ht'.
-			' WHERE ht.hostid='.zbx_dbstr($templateId)
-		);
-		while ($dbHttpTest = DBfetch($dbCursor)) {
-			$httpTests[$dbHttpTest['httptestid']] = $dbHttpTest;
-		}
-
-		$dbCursor = DBselect(
-			'SELECT hs.httpstepid,hs.httptestid,hs.name,hs.no,hs.url,hs.timeout,hs.posts,hs.variables,'.
-				'hs.required,hs.status_codes,hs.headers,hs.follow_redirects,hs.retrieve_mode'.
-			' FROM httpstep hs'.
-			' WHERE '.dbConditionInt('hs.httptestid', array_keys($httpTests))
-		);
-		while ($dbHttpStep = DBfetch($dbCursor)) {
-			$httpTests[$dbHttpStep['httptestid']]['steps'][] = $dbHttpStep;
-		}
+		$httpTests = API::HttpTest()->get([
+			'output' => ['httptestid', 'name', 'applicationid', 'delay', 'status', 'agent', 'authentication',
+				'http_user', 'http_password', 'hostid', 'templateid', 'http_proxy', 'retries', 'ssl_cert_file',
+				'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host', 'variables', 'headers'
+			],
+			'hostids' => $templateId,
+			'selectSteps' => ['httpstepid', 'name', 'no', 'url', 'timeout', 'posts', 'required', 'status_codes',
+				'follow_redirects', 'retrieve_mode', 'variables', 'headers', 'query_fields'
+			],
+			'preservekeys' => true
+		]);
 
 		$this->inherit($httpTests, $hostIds);
 	}
@@ -401,23 +380,32 @@ class CHttpTestManager {
 				if (isset($hostHttpTest['byTemplateId'][$httpTestId])) {
 					$exHttpTest = $hostHttpTest['byTemplateId'][$httpTestId];
 
-					// need to check templateid here too in case we update linked http test to name that already exists on linked host
+					/*
+					 * 'templateid' needs to be checked here too in case we update linked httptest to name
+					 * that already exists on a linked host.
+					 */
 					if (isset($httpTest['name']) && isset($hostHttpTest['byName'][$httpTest['name']])
 							&& !idcmp($exHttpTest['templateid'], $hostHttpTest['byName'][$httpTest['name']]['templateid'])) {
 						$host = DBfetch(DBselect('SELECT h.name FROM hosts h WHERE h.hostid='.zbx_dbstr($hostId)));
-						throw new Exception(_s('Web scenario "%1$s" already exists on host "%2$s".', $exHttpTest['name'], $host['name']));
+						throw new Exception(
+							_s('Web scenario "%1$s" already exists on host "%2$s".', $exHttpTest['name'], $host['name'])
+						);
 					}
 				}
 				// update by name
-				else if (isset($hostHttpTest['byName'][$httpTest['name']])) {
+				elseif (isset($hostHttpTest['byName'][$httpTest['name']])) {
 					$exHttpTest = $hostHttpTest['byName'][$httpTest['name']];
-					if ($exHttpTest['templateid'] > 0 || !$this->compareHttpSteps($httpTest, $exHttpTest)) {
-						$host = DBfetch(DBselect('SELECT h.name FROM hosts h WHERE h.hostid='.zbx_dbstr($hostId)));
-						throw new Exception(_s('Web scenario "%1$s" already exists on host "%2$s".', $exHttpTest['name'], $host['name']));
-					}
 
-					$this->createLinkageBetweenHttpTests($httpTestId, $exHttpTest['httptestid']);
-					continue;
+					if (bccomp($exHttpTest['templateid'], $httpTestId) == 0 || !$this->compareHttpSteps($httpTest, $exHttpTest)) {
+						$host = DBfetch(DBselect('SELECT h.name FROM hosts h WHERE h.hostid='.zbx_dbstr($hostId)));
+						throw new Exception(
+							_s('Web scenario "%1$s" already exists on host "%2$s".', $exHttpTest['name'], $host['name'])
+						);
+					}
+					elseif ($this->compareHttpProperties($httpTest, $exHttpTest)) {
+						$this->createLinkageBetweenHttpTests($httpTestId, $exHttpTest['httptestid']);
+						continue;
+					}
 				}
 
 				$newHttpTest = $httpTest;
@@ -447,6 +435,22 @@ class CHttpTestManager {
 		return $result;
 	}
 
+	/**
+	 * Compare properties for http tests.
+	 *
+	 * @param array $http_test			Current http test properties.
+	 * @param array $ex_http_test		Existing http test properties to compare with.
+	 *
+	 * @return bool
+	 */
+	protected function compareHttpProperties(array $http_test, array $ex_http_test) {
+		return ($http_test['http_proxy'] === $ex_http_test['http_proxy']
+				&& $http_test['agent'] === $ex_http_test['agent']
+				&& $http_test['retries'] == $ex_http_test['retries']
+				&& $http_test['delay'] === $ex_http_test['delay']
+				&& bccomp($http_test['applicationid'], $ex_http_test['applicationid']) == 0);
+	}
+
 	/**
 	 * Create linkage between two http tests.
 	 * If we found existing http test by name and steps, we only add linkage, i.e. change templateid
@@ -551,7 +555,8 @@ class CHttpTestManager {
 		}
 
 		$dbCursor = DBselect(
-			'SELECT ht.httptestid,ht.name,ht.hostid,ht.templateid'.
+			'SELECT ht.httptestid,ht.name,ht.applicationid,ht.delay,ht.agent,ht.hostid,ht.templateid,ht.http_proxy,'.
+				'ht.retries'.
 			' FROM httptest ht'.
 			' WHERE '.dbConditionInt('ht.hostid', $hostIds)
 		);
@@ -599,34 +604,47 @@ class CHttpTestManager {
 	/**
 	 * Save http tests. If http test has httptestid it gets updated otherwise a new one is created.
 	 *
-	 * @param array $httpTests
+	 * @param array $http_tests
 	 *
 	 * @return array
 	 */
-	protected function save(array $httpTests) {
-		$httpTestsCreate = [];
-		$httpTestsUpdate = [];
+	protected function save(array $http_tests) {
+		$http_tests_to_create = [];
+		$http_tests_to_update = [];
 
-		foreach ($httpTests as $httpTest) {
-			if (isset($httpTest['httptestid'])) {
-				$httpTestsUpdate[] = $httpTest;
+		foreach ($http_tests as $num => $http_test) {
+			if (array_key_exists('httptestid', $http_test)) {
+				$http_tests_to_update[] = $http_test;
 			}
 			else {
-				$httpTestsCreate[] = $httpTest;
+				$http_tests_to_create[] = $http_test;
+			}
+
+			/*
+			 * Unset $http_tests and (later) put it back with actual httptestid as a key right after creating/updateing
+			 * it. This is done in such a way because $http_tests array holds items with incremental keys which are not
+			 * a real httptestids.
+			 */
+			unset($http_tests[$num]);
+		}
+
+		if ($http_tests_to_create) {
+			$new_http_tests = $this->create($http_tests_to_create);
+
+			foreach ($new_http_tests as $new_http_test) {
+				$http_tests[$new_http_test['httptestid']] = $new_http_test;
 			}
 		}
 
-		if (!empty($httpTestsCreate)) {
-			$newHttpTests = $this->create($httpTestsCreate);
-			foreach ($newHttpTests as $num => $newHttpTest) {
-				$httpTests[$num]['httptestid'] = $newHttpTest['httptestid'];
+		if ($http_tests_to_update) {
+			$updated_http_tests = $this->update($http_tests_to_update);
+
+			foreach ($updated_http_tests as $updated_http_test) {
+				$http_tests[$updated_http_test['httptestid']] = $updated_http_test;
 			}
 		}
-		if (!empty($httpTestsUpdate)) {
-			$this->update($httpTestsUpdate);
-		}
 
-		return $httpTests;
+		return $http_tests;
 	}
 
 	/**
@@ -712,14 +730,16 @@ class CHttpTestManager {
 
 		$insertItems = [];
 
+		$delay = array_key_exists('delay', $httpTest) ? $httpTest['delay'] : DB::getDefault('httptest', 'delay');
+		$status = array_key_exists('status', $httpTest) ? $httpTest['status'] : DB::getDefault('httptest', 'status');
+
 		foreach ($checkitems as $item) {
-			$item['data_type'] = ITEM_DATA_TYPE_DECIMAL;
 			$item['hostid'] = $httpTest['hostid'];
-			$item['delay'] = $httpTest['delay'];
+			$item['delay'] = $delay;
 			$item['type'] = ITEM_TYPE_HTTPTEST;
 			$item['history'] = self::ITEM_HISTORY;
 			$item['trends'] = self::ITEM_TRENDS;
-			$item['status'] = (HTTPTEST_STATUS_ACTIVE == $httpTest['status'])
+			$item['status'] = ($status == HTTPTEST_STATUS_ACTIVE)
 				? ITEM_STATUS_ACTIVE
 				: ITEM_STATUS_DISABLED;
 
@@ -747,6 +767,198 @@ class CHttpTestManager {
 		DB::insert('httptestitem', $httpTestItems);
 	}
 
+	/**
+	 * Create web scenario fields.
+	 *
+	 * @param array  $httptests
+	 * @param string $httptests['httptestid']
+	 * @param array  $httptests['variables']           (optional)
+	 * @param string $httptests['variables']['name']
+	 * @param string $httptests['variables']['value']
+	 * @param array  $httptests['headers']             (optional)
+	 * @param string $httptests['headers']['name']
+	 * @param string $httptests['headers']['value']
+	 * @param string $method
+	 */
+	private function updateHttpTestFields(array $httptests, $method) {
+		$fields = [
+			ZBX_HTTPFIELD_VARIABLE => 'variables',
+			ZBX_HTTPFIELD_HEADER => 'headers'
+		];
+		$httptest_fields = [];
+
+		foreach ($httptests as $httptest) {
+			foreach ($fields as $type => $field) {
+				if (array_key_exists($field, $httptest)) {
+					$httptest_fields[$httptest['httptestid']][$type] = $httptest[$field];
+				}
+			}
+		}
+
+		if (!$httptest_fields) {
+			return;
+		}
+
+		$db_httptest_fields = ($method === 'update')
+			? DB::select('httptest_field', [
+				'output' => ['httptest_fieldid', 'httptestid', 'type', 'name', 'value'],
+				'filter' => ['httptestid' => array_keys($httptest_fields)],
+				'sortfield' => ['httptest_fieldid']
+			])
+			: [];
+
+		$ins_httptest_fields = [];
+		$upd_httptest_fields = [];
+		$del_httptest_fieldids = [];
+
+		foreach ($db_httptest_fields as $index =>  $db_httptest_field) {
+			if (array_key_exists($db_httptest_field['type'], $httptest_fields[$db_httptest_field['httptestid']])) {
+				$httptest_field =
+					array_shift($httptest_fields[$db_httptest_field['httptestid']][$db_httptest_field['type']]);
+
+				if ($httptest_field !== null) {
+					$upd_httptest_field = [];
+
+					foreach (['name', 'value'] as $field_name) {
+						if ($httptest_field[$field_name] !== $db_httptest_field[$field_name]) {
+							$upd_httptest_field[$field_name] = $httptest_field[$field_name];
+						}
+					}
+
+					if ($upd_httptest_field) {
+						$upd_httptest_fields[] = [
+							'values' => $upd_httptest_field,
+							'where' => ['httptest_fieldid' => $db_httptest_field['httptest_fieldid']]
+						];
+					}
+				}
+				else {
+					$del_httptest_fieldids[] = $db_httptest_field['httptest_fieldid'];
+				}
+			}
+		}
+
+		foreach ($httptest_fields as $httptestid => $httptest_fields_by_httptest) {
+			foreach ($httptest_fields_by_httptest as $type => $httptest_fields_by_type) {
+				foreach ($httptest_fields_by_type as $httptest_field) {
+					$ins_httptest_fields[] = [
+						'httptestid' => $httptestid,
+						'type' => $type
+					] + $httptest_field;
+				}
+			}
+		}
+
+		if ($ins_httptest_fields) {
+			DB::insertBatch('httptest_field', $ins_httptest_fields);
+		}
+
+		if ($upd_httptest_fields) {
+			DB::update('httptest_field', $upd_httptest_fields);
+		}
+
+		if ($del_httptest_fieldids) {
+			DB::delete('httptest_field', ['httptest_fieldid' => $del_httptest_fieldids]);
+		}
+	}
+
+	/**
+	 * Create web scenario step fields.
+	 *
+	 * @param array  $httpsteps
+	 * @param string $httpsteps['httpstepid']
+	 * @param array  $httpsteps['variables']           (optional)
+	 * @param string $httpsteps['variables']['name']
+	 * @param string $httpsteps['variables']['value']
+	 * @param array  $httpsteps['headers']             (optional)
+	 * @param string $httpsteps['headers']['name']
+	 * @param string $httpsteps['headers']['value']
+	 * @param string $method
+	 */
+	private function updateHttpStepFields(array $httpsteps, $method) {
+		$fields = [
+			ZBX_HTTPFIELD_VARIABLE => 'variables',
+			ZBX_HTTPFIELD_HEADER => 'headers',
+			ZBX_HTTPFIELD_POST_FIELD => 'post_fields',
+			ZBX_HTTPFIELD_QUERY_FIELD => 'query_fields'
+		];
+		$httpstep_fields = [];
+
+		foreach ($httpsteps as $httpstep) {
+			foreach ($fields as $type => $field) {
+				if (array_key_exists($field, $httpstep)) {
+					$httpstep_fields[$httpstep['httpstepid']][$type] = $httpstep[$field];
+				}
+			}
+		}
+
+		if (!$httpstep_fields) {
+			return;
+		}
+
+		$db_httpstep_fields = ($method === 'update')
+			? DB::select('httpstep_field', [
+				'output' => ['httpstep_fieldid', 'httpstepid', 'type', 'name', 'value'],
+				'filter' => ['httpstepid' => array_keys($httpstep_fields)],
+				'sortfield' => ['httpstep_fieldid']
+			])
+			: [];
+
+		$ins_httpstep_fields = [];
+		$upd_httpstep_fields = [];
+		$del_httpstep_fieldids = [];
+
+		foreach ($db_httpstep_fields as $index =>  $db_httpstep_field) {
+			if (array_key_exists($db_httpstep_field['type'], $httpstep_fields[$db_httpstep_field['httpstepid']])) {
+				$httpstep_field =
+					array_shift($httpstep_fields[$db_httpstep_field['httpstepid']][$db_httpstep_field['type']]);
+
+				if ($httpstep_field !== null) {
+					$upd_httpstep_field = [];
+
+					foreach (['name', 'value'] as $field_name) {
+						if ($httpstep_field[$field_name] !== $db_httpstep_field[$field_name]) {
+							$upd_httpstep_field[$field_name] = $httpstep_field[$field_name];
+						}
+					}
+
+					if ($upd_httpstep_field) {
+						$upd_httpstep_fields[] = [
+							'values' => $upd_httpstep_field,
+							'where' => ['httpstep_fieldid' => $db_httpstep_field['httpstep_fieldid']]
+						];
+					}
+				}
+				else {
+					$del_httpstep_fieldids[] = $db_httpstep_field['httpstep_fieldid'];
+				}
+			}
+		}
+
+		foreach ($httpstep_fields as $httpstepid => $httpstep_fields_by_httpstep) {
+			foreach ($httpstep_fields_by_httpstep as $type => $httpstep_fields_by_type) {
+				foreach ($httpstep_fields_by_type as $httpstep_field) {
+					$ins_httpstep_fields[] = [
+						'httpstepid' => $httpstepid,
+						'type' => $type
+					] + $httpstep_field;
+				}
+			}
+		}
+
+		if ($ins_httpstep_fields) {
+			DB::insertBatch('httpstep_field', $ins_httpstep_fields);
+		}
+
+		if ($upd_httpstep_fields) {
+			DB::update('httpstep_field', $upd_httpstep_fields);
+		}
+
+		if ($del_httpstep_fieldids) {
+			DB::delete('httpstep_field', ['httpstep_fieldid' => $del_httpstep_fieldids]);
+		}
+	}
+
 	/**
 	 * Create web scenario steps with items.
 	 *
@@ -756,9 +968,23 @@ class CHttpTestManager {
 	 * @throws Exception
 	 */
 	protected function createStepsReal($httpTest, $websteps) {
-		foreach ($websteps as $snum => $webstep) {
-			$websteps[$snum]['httptestid'] = $httpTest['httptestid'];
+		foreach ($websteps as &$webstep) {
+			$webstep['httptestid'] = $httpTest['httptestid'];
+
+			if (array_key_exists('posts', $webstep)) {
+				if (is_array($webstep['posts'])) {
+					$webstep['post_fields'] = $webstep['posts'];
+					$webstep['posts'] = '';
+					$webstep['post_type'] = ZBX_POSTTYPE_FORM;
+				}
+				else {
+					$webstep['post_fields'] = [];
+					$webstep['post_type'] = ZBX_POSTTYPE_RAW;
+				}
+			}
 		}
+		unset($webstep);
+
 		$webstepids = DB::insert('httpstep', $websteps);
 
 		// if this is a template scenario, fetch the parent http items to link inherited items to them
@@ -773,8 +999,8 @@ class CHttpTestManager {
 			), 'key_');
 		}
 
-		foreach ($websteps as $snum => $webstep) {
-			$webstepid = $webstepids[$snum];
+		foreach ($websteps as $snum => &$webstep) {
+			$webstep['httpstepid'] = $webstepids[$snum];
 
 			$stepitems = [
 				[
@@ -817,7 +1043,6 @@ class CHttpTestManager {
 				$item['hostid'] = $httpTest['hostid'];
 				$item['delay'] = $delay;
 				$item['type'] = ITEM_TYPE_HTTPTEST;
-				$item['data_type'] = ITEM_DATA_TYPE_DECIMAL;
 				$item['history'] = self::ITEM_HISTORY;
 				$item['trends'] = self::ITEM_TRENDS;
 				$item['status'] = (HTTPTEST_STATUS_ACTIVE == $status) ? ITEM_STATUS_ACTIVE : ITEM_STATUS_DISABLED;
@@ -840,13 +1065,16 @@ class CHttpTestManager {
 			$webstepitems = [];
 			foreach ($stepitems as $inum => $item) {
 				$webstepitems[] = [
-					'httpstepid' => $webstepid,
+					'httpstepid' => $webstep['httpstepid'],
 					'itemid' => $stepItemids[$inum],
 					'type' => $item['httpstepitemtype']
 				];
 			}
 			DB::insert('httpstepitem', $webstepitems);
 		}
+		unset($webstep);
+
+		$this->updateHttpStepFields($websteps, 'create');
 	}
 
 	/**
@@ -870,6 +1098,21 @@ class CHttpTestManager {
 			, 'key_'
 		);
 
+		foreach ($websteps as &$webstep) {
+			if (array_key_exists('posts', $webstep)) {
+				if (is_array($webstep['posts'])) {
+					$webstep['post_fields'] = $webstep['posts'];
+					$webstep['posts'] = '';
+					$webstep['post_type'] = ZBX_POSTTYPE_FORM;
+				}
+				else {
+					$webstep['post_fields'] = [];
+					$webstep['post_type'] = ZBX_POSTTYPE_RAW;
+				}
+			}
+		}
+		unset($webstep);
+
 		foreach ($websteps as $webstep) {
 			DB::update('httpstep', [
 				'values' => $webstep,
@@ -926,6 +1169,8 @@ class CHttpTestManager {
 				$this->updateItemsApplications($itemids, $httpTest['applicationid']);
 			}
 		}
+
+		$this->updateHttpStepFields($websteps, 'update');
 	}
 
 	/**
@@ -969,7 +1214,7 @@ class CHttpTestManager {
 				$insert[] = ['itemid' => $itemid, 'applicationid' => $applicationid];
 			}
 
-			DB::insert('items_applications', $insert);
+			DB::insertBatch('items_applications', $insert);
 		}
 	}
 
@@ -1024,7 +1269,7 @@ class CHttpTestManager {
 	 * - lastfailedstep - number of the last failed step
 	 * - error          - error message
 	 *
-	 * If a HTTP test has never been executed, no value will be returned.
+	 * If a HTTP test has not been executed in last ZBX_HISTORY_PERIOD, no value will be returned.
 	 *
 	 * @param array $httpTestIds
 	 *
@@ -1039,7 +1284,7 @@ class CHttpTestManager {
 				' AND '.dbConditionInt('hti.httptestid', $httpTestIds)
 		));
 
-		$history = Manager::History()->getLast($httpItems);
+		$history = Manager::History()->getLastValues($httpItems, 1, ZBX_HISTORY_PERIOD);
 
 		$data = [];
 
