@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ class CEvent extends CApiService {
 	 * @param array $options['eventids']
 	 * @param array $options['applicationids']
 	 * @param array $options['status']
-	 * @param array $options['editable']
+	 * @param bool  $options['editable']
 	 * @param array $options['count']
 	 * @param array $options['pattern']
 	 * @param array $options['limit']
@@ -69,12 +69,10 @@ class CEvent extends CApiService {
 	 */
 	public function get($options = []) {
 		$result = [];
-		$userType = self::$userData['type'];
-		$userid = self::$userData['userid'];
 
 		$sqlParts = [
 			'select'	=> [$this->fieldId('eventid')],
-			'from'		=> ['events' => 'events e'],
+			'from'		=> ['e' => 'events e'],
 			'where'		=> [],
 			'order'		=> [],
 			'group'		=> [],
@@ -88,7 +86,7 @@ class CEvent extends CApiService {
 			'applicationids'			=> null,
 			'objectids'					=> null,
 
-			'editable'					=> null,
+			'editable'					=> false,
 			'object'					=> EVENT_OBJECT_TRIGGER,
 			'source'					=> EVENT_SOURCE_TRIGGERS,
 			'severities'				=> null,
@@ -105,8 +103,8 @@ class CEvent extends CApiService {
 			'filter'					=> null,
 			'search'					=> null,
 			'searchByAny'				=> null,
-			'startSearch'				=> null,
-			'excludeSearch'				=> null,
+			'startSearch'				=> false,
+			'excludeSearch'				=> false,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
@@ -115,9 +113,9 @@ class CEvent extends CApiService {
 			'select_alerts'				=> null,
 			'select_acknowledges'		=> null,
 			'selectTags'				=> null,
-			'countOutput'				=> null,
-			'groupCount'				=> null,
-			'preservekeys'				=> null,
+			'countOutput'				=> false,
+			'groupCount'				=> false,
+			'preservekeys'				=> false,
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null
@@ -131,7 +129,7 @@ class CEvent extends CApiService {
 		$sqlParts['where'][] = 'e.object='.zbx_dbstr($options['object']);
 
 		// editable + PERMISSION CHECK
-		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			// triggers
 			if ($options['object'] == EVENT_OBJECT_TRIGGER) {
 				// specific triggers
@@ -145,19 +143,21 @@ class CEvent extends CApiService {
 				}
 				// all triggers
 				else {
-					$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
-					$sqlParts['where'][] = 'EXISTS ('.
+					$userGroups = getUserGroupsByUserId(self::$userData['userid']);
+
+					$sqlParts['where'][] = 'NOT EXISTS ('.
 							'SELECT NULL'.
 							' FROM functions f,items i,hosts_groups hgg'.
-								' JOIN rights r'.
+								' LEFT JOIN rights r'.
 									' ON r.id=hgg.groupid'.
-										' AND '.dbConditionInt('r.groupid', getUserGroupsByUserId($userid)).
+										' AND '.dbConditionInt('r.groupid', $userGroups).
 							' WHERE e.objectid=f.triggerid'.
 								' AND f.itemid=i.itemid'.
 								' AND i.hostid=hgg.hostid'.
-							' GROUP BY f.triggerid'.
-							' HAVING MIN(r.permission)>'.PERM_DENY.
-								' AND MAX(r.permission)>='.zbx_dbstr($permission).
+							' GROUP BY i.hostid'.
+							' HAVING MAX(permission)<'.($options['editable'] ? PERM_READ_WRITE : PERM_READ).
+								' OR MIN(permission) IS NULL'.
+								' OR MIN(permission)='.PERM_DENY.
 							')';
 				}
 			}
@@ -184,18 +184,19 @@ class CEvent extends CApiService {
 				}
 				// all items and LLD rules
 				else {
-					$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
+					$userGroups = getUserGroupsByUserId(self::$userData['userid']);
+
 					$sqlParts['where'][] = 'EXISTS ('.
 							'SELECT NULL'.
 							' FROM items i,hosts_groups hgg'.
 								' JOIN rights r'.
 									' ON r.id=hgg.groupid'.
-										' AND '.dbConditionInt('r.groupid', getUserGroupsByUserId($userid)).
+										' AND '.dbConditionInt('r.groupid', $userGroups).
 							' WHERE e.objectid=i.itemid'.
 								' AND i.hostid=hgg.hostid'.
 							' GROUP BY hgg.hostid'.
 							' HAVING MIN(r.permission)>'.PERM_DENY.
-								' AND MAX(r.permission)>='.zbx_dbstr($permission).
+								' AND MAX(r.permission)>='.($options['editable'] ? PERM_READ_WRITE : PERM_READ).
 							')';
 				}
 			}
@@ -214,34 +215,32 @@ class CEvent extends CApiService {
 			zbx_value2array($options['objectids']);
 			$sqlParts['where'][] = dbConditionInt('e.objectid', $options['objectids']);
 
-			if (!is_null($options['groupCount'])) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['objectid'] = 'e.objectid';
 			}
 		}
 
-		$sub_sql_parts = [];
-
 		// groupids
 		if ($options['groupids'] !== null) {
 			zbx_value2array($options['groupids']);
 
 			// triggers
 			if ($options['object'] == EVENT_OBJECT_TRIGGER) {
-				$sub_sql_parts['from']['f'] = 'functions f';
-				$sub_sql_parts['from']['i'] = 'items i';
-				$sub_sql_parts['from']['hg'] = 'hosts_groups hg';
-				$sub_sql_parts['where']['e-f'] = 'e.objectid=f.triggerid';
-				$sub_sql_parts['where']['f-i'] = 'f.itemid=i.itemid';
-				$sub_sql_parts['where']['i-hg'] = 'i.hostid=hg.hostid';
-				$sub_sql_parts['where']['hg'] = dbConditionInt('hg.groupid', $options['groupids']);
+				$sqlParts['from']['f'] = 'functions f';
+				$sqlParts['from']['i'] = 'items i';
+				$sqlParts['from']['hg'] = 'hosts_groups hg';
+				$sqlParts['where']['e-f'] = 'e.objectid=f.triggerid';
+				$sqlParts['where']['f-i'] = 'f.itemid=i.itemid';
+				$sqlParts['where']['i-hg'] = 'i.hostid=hg.hostid';
+				$sqlParts['where']['hg'] = dbConditionInt('hg.groupid', $options['groupids']);
 			}
 			// lld rules and items
 			elseif ($options['object'] == EVENT_OBJECT_LLDRULE || $options['object'] == EVENT_OBJECT_ITEM) {
-				$sub_sql_parts['from']['i'] = 'items i';
-				$sub_sql_parts['from']['hg'] = 'hosts_groups hg';
-				$sub_sql_parts['where']['e-i'] = 'e.objectid=i.itemid';
-				$sub_sql_parts['where']['i-hg'] = 'i.hostid=hg.hostid';
-				$sub_sql_parts['where']['hg'] = dbConditionInt('hg.groupid', $options['groupids']);
+				$sqlParts['from']['i'] = 'items i';
+				$sqlParts['from']['hg'] = 'hosts_groups hg';
+				$sqlParts['where']['e-i'] = 'e.objectid=i.itemid';
+				$sqlParts['where']['i-hg'] = 'i.hostid=hg.hostid';
+				$sqlParts['where']['hg'] = dbConditionInt('hg.groupid', $options['groupids']);
 			}
 		}
 
@@ -251,17 +250,17 @@ class CEvent extends CApiService {
 
 			// triggers
 			if ($options['object'] == EVENT_OBJECT_TRIGGER) {
-				$sub_sql_parts['from']['f'] = 'functions f';
-				$sub_sql_parts['from']['i'] = 'items i';
-				$sub_sql_parts['where']['e-f'] = 'e.objectid=f.triggerid';
-				$sub_sql_parts['where']['f-i'] = 'f.itemid=i.itemid';
-				$sub_sql_parts['where']['i'] = dbConditionInt('i.hostid', $options['hostids']);
+				$sqlParts['from']['f'] = 'functions f';
+				$sqlParts['from']['i'] = 'items i';
+				$sqlParts['where']['e-f'] = 'e.objectid=f.triggerid';
+				$sqlParts['where']['f-i'] = 'f.itemid=i.itemid';
+				$sqlParts['where']['i'] = dbConditionInt('i.hostid', $options['hostids']);
 			}
 			// lld rules and items
 			elseif ($options['object'] == EVENT_OBJECT_LLDRULE || $options['object'] == EVENT_OBJECT_ITEM) {
-				$sub_sql_parts['from']['i'] = 'items i';
-				$sub_sql_parts['where']['e-i'] = 'e.objectid=i.itemid';
-				$sub_sql_parts['where']['i'] = dbConditionInt('i.hostid', $options['hostids']);
+				$sqlParts['from']['i'] = 'items i';
+				$sqlParts['where']['e-i'] = 'e.objectid=i.itemid';
+				$sqlParts['where']['i'] = dbConditionInt('i.hostid', $options['hostids']);
 			}
 		}
 
@@ -271,17 +270,17 @@ class CEvent extends CApiService {
 
 			// triggers
 			if ($options['object'] == EVENT_OBJECT_TRIGGER) {
-				$sub_sql_parts['from']['f'] = 'functions f';
-				$sub_sql_parts['from']['ia'] = 'items_applications ia';
-				$sub_sql_parts['where']['e-f'] = 'e.objectid=f.triggerid';
-				$sub_sql_parts['where']['f-ia'] = 'f.itemid=ia.itemid';
-				$sub_sql_parts['where']['ia'] = dbConditionInt('ia.applicationid', $options['applicationids']);
+				$sqlParts['from']['f'] = 'functions f';
+				$sqlParts['from']['ia'] = 'items_applications ia';
+				$sqlParts['where']['e-f'] = 'e.objectid=f.triggerid';
+				$sqlParts['where']['f-ia'] = 'f.itemid=ia.itemid';
+				$sqlParts['where']['ia'] = dbConditionInt('ia.applicationid', $options['applicationids']);
 			}
 			// items
 			elseif ($options['object'] == EVENT_OBJECT_ITEM) {
-				$sub_sql_parts['from']['ia'] = 'items_applications ia';
-				$sub_sql_parts['where']['e-ia'] = 'e.objectid=ia.itemid';
-				$sub_sql_parts['where']['ia'] = dbConditionInt('ia.applicationid', $options['applicationids']);
+				$sqlParts['from']['ia'] = 'items_applications ia';
+				$sqlParts['where']['e-ia'] = 'e.objectid=ia.itemid';
+				$sqlParts['where']['ia'] = dbConditionInt('ia.applicationid', $options['applicationids']);
 			}
 			// ignore this filter for lld rules
 		}
@@ -292,21 +291,13 @@ class CEvent extends CApiService {
 
 			// triggers
 			if ($options['object'] == EVENT_OBJECT_TRIGGER) {
-				$sub_sql_parts['from']['t'] = 'triggers t';
-				$sub_sql_parts['where']['e-t'] = 'e.objectid=t.triggerid';
-				$sub_sql_parts['where']['t'] = dbConditionInt('t.priority', $options['severities']);
+				$sqlParts['from']['t'] = 'triggers t';
+				$sqlParts['where']['e-t'] = 'e.objectid=t.triggerid';
+				$sqlParts['where']['t'] = dbConditionInt('t.priority', $options['severities']);
 			}
 			// ignore this filter for items and lld rules
 		}
 
-		if ($sub_sql_parts) {
-			$sqlParts['where'][] = 'EXISTS ('.
-				'SELECT NULL'.
-				' FROM '.implode(',', $sub_sql_parts['from']).
-				' WHERE '.implode(' AND ', array_unique($sub_sql_parts['where'])).
-			')';
-		}
-
 		// acknowledged
 		if (!is_null($options['acknowledged'])) {
 			$sqlParts['where'][] = 'e.acknowledged='.($options['acknowledged'] ? 1 : 0);
@@ -378,8 +369,8 @@ class CEvent extends CApiService {
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($event = DBfetch($res)) {
-			if (!is_null($options['countOutput'])) {
-				if (!is_null($options['groupCount'])) {
+			if ($options['countOutput']) {
+				if ($options['groupCount']) {
 					$result[] = $event;
 				}
 				else {
@@ -391,7 +382,7 @@ class CEvent extends CApiService {
 			}
 		}
 
-		if (!is_null($options['countOutput'])) {
+		if ($options['countOutput']) {
 			return $result;
 		}
 
@@ -401,7 +392,7 @@ class CEvent extends CApiService {
 		}
 
 		// removing keys (hash -> array)
-		if (is_null($options['preservekeys'])) {
+		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
 
@@ -476,14 +467,19 @@ class CEvent extends CApiService {
 
 		$acknowledgeids = DB::insert('acknowledges', $acknowledges);
 
+		$ack_count = count($acknowledgeids);
+
 		if ($action == ZBX_ACKNOWLEDGE_ACTION_CLOSE_PROBLEM) {
 			// Close the problem manually.
 
 			$tasks = [];
-			$ack_count = count($acknowledgeids);
 
 			for ($i = 0; $i < $ack_count; $i++) {
-				$tasks[] = ['type' => ZBX_TM_TASK_CLOSE_PROBLEM];
+				$tasks[] = [
+					'type' => ZBX_TM_TASK_CLOSE_PROBLEM,
+					'status' => ZBX_TM_STATUS_NEW,
+					'clock' => $time
+				];
 			}
 
 			$taskids = DB::insert('task', $tasks);
@@ -500,6 +496,29 @@ class CEvent extends CApiService {
 			DB::insert('task_close_problem', $task_close, false);
 		}
 
+		$tasks = [];
+
+		for ($i = 0; $i < $ack_count; $i++) {
+			$tasks[] = [
+				'type' => ZBX_TM_TASK_ACKNOWLEDGE,
+				'status' => ZBX_TM_STATUS_NEW,
+				'clock' => $time
+			];
+		}
+
+		$taskids = DB::insert('task', $tasks);
+
+		$tasks_ack = [];
+
+		for ($i = 0; $i < $ack_count; $i++) {
+			$tasks_ack[] = [
+				'taskid' => $taskids[$i],
+				'acknowledgeid' => $acknowledgeids[$i]
+			];
+		}
+
+		DB::insert('task_acknowledge', $tasks_ack, false);
+
 		return ['eventids' => array_values($eventids)];
 	}
 
@@ -542,16 +561,20 @@ class CEvent extends CApiService {
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
-		if ($options['countOutput'] === null) {
-			if ($this->outputIsRequested('r_eventid', $options['output'])
-					|| $this->outputIsRequested('c_eventid', $options['output'])
+		if (!$options['countOutput']) {
+			if ($this->outputIsRequested('r_eventid', $options['output'])) {
+				// Select fields from event_recovery table using LEFT JOIN.
+
+				$sqlParts['select']['r_eventid'] = 'er1.r_eventid';
+				$sqlParts['left_join'][] = ['from' => 'event_recovery er1', 'on' => 'er1.eventid=e.eventid'];
+				$sqlParts['left_table'] = 'e';
+			}
+
+			if ($this->outputIsRequested('c_eventid', $options['output'])
 					|| $this->outputIsRequested('correlationid', $options['output'])
 					|| $this->outputIsRequested('userid', $options['output'])) {
 				// Select fields from event_recovery table using LEFT JOIN.
 
-				if ($this->outputIsRequested('r_eventid', $options['output'])) {
-					$sqlParts['select']['r_eventid'] = 'er1.r_eventid';
-				}
 				if ($this->outputIsRequested('c_eventid', $options['output'])) {
 					$sqlParts['select']['c_eventid'] = 'er2.c_eventid';
 				}
@@ -562,8 +585,8 @@ class CEvent extends CApiService {
 					$sqlParts['select']['userid'] = 'er2.userid';
 				}
 
-				$sqlParts['left_join'][] = ['from' => 'event_recovery er1', 'on' => 'er1.eventid=e.eventid'];
 				$sqlParts['left_join'][] = ['from' => 'event_recovery er2', 'on' => 'er2.r_eventid=e.eventid'];
+				$sqlParts['left_table'] = 'e';
 			}
 
 			if ($options['selectRelatedObject'] !== null || $options['selectHosts'] !== null) {
@@ -736,15 +759,28 @@ class CEvent extends CApiService {
 
 		// Adding event tags.
 		if ($options['selectTags'] !== null && $options['selectTags'] != API_OUTPUT_COUNT) {
-			$tags = API::getApiService()->select('event_tag', [
-				'output' => $this->outputExtend($options['selectTags'], ['eventid']),
-				'filter' => ['eventid' => $eventIds],
-				'preservekeys' => true
-			]);
+			if ($options['selectTags'] === API_OUTPUT_EXTEND) {
+				$options['selectTags'] = ['tag', 'value'];
+			}
 
-			$relationMap = $this->createRelationMap($tags, 'eventid', 'eventtagid');
-			$tags = $this->unsetExtraFields($tags, ['eventtagid', 'eventid'], []);
-			$result = $relationMap->mapMany($result, $tags, 'tags');
+			$tags_options = [
+				'output' => $this->outputExtend($options['selectTags'], ['eventid']),
+				'filter' => ['eventid' => $eventIds]
+			];
+			$tags = DBselect(DB::makeSql('event_tag', $tags_options));
+
+			foreach ($result as &$event) {
+				$event['tags'] = [];
+			}
+			unset($event);
+
+			while ($tag = DBfetch($tags)) {
+				$event = &$result[$tag['eventid']];
+
+				unset($tag['eventtagid'], $tag['eventid']);
+				$event['tags'][] = $tag;
+			}
+			unset($event);
 		}
 
 		return $result;
