@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,8 +21,6 @@
 
 /**
  * Class containing methods for operations with users media.
- *
- * @package API
  */
 class CUserMedia extends CApiService {
 
@@ -33,6 +31,8 @@ class CUserMedia extends CApiService {
 	/**
 	 * Get users data.
 	 *
+	 * @deprecated	As of version 3.4, use user.get() method instead.
+	 *
 	 * @param array  $options
 	 * @param array  $options['usrgrpids']	filter by UserGroup IDs
 	 * @param array  $options['userids']	filter by User IDs
@@ -47,6 +47,8 @@ class CUserMedia extends CApiService {
 	 * @return array
 	 */
 	public function get($options = []) {
+		$this->deprecated('usermedia.get method is deprecated.');
+
 		$result = [];
 
 		$sqlParts = [
@@ -67,15 +69,15 @@ class CUserMedia extends CApiService {
 			'filter'					=> null,
 			'search'					=> null,
 			'searchByAny'				=> null,
-			'startSearch'				=> null,
-			'excludeSearch'				=> null,
+			'startSearch'				=> false,
+			'excludeSearch'				=> false,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
-			'editable'					=> null,
-			'countOutput'				=> null,
-			'groupCount'				=> null,
-			'preservekeys'				=> null,
+			'editable'					=> false,
+			'countOutput'				=> false,
+			'groupCount'				=> false,
+			'preservekeys'				=> false,
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null
@@ -114,7 +116,7 @@ class CUserMedia extends CApiService {
 			$sqlParts['where'][] = dbConditionInt('u.userid', $options['userids']);
 			$sqlParts['where']['mu'] = 'm.userid=u.userid';
 
-			if ($options['groupCount'] !== null) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['userid'] = 'm.userid';
 			}
 		}
@@ -127,7 +129,7 @@ class CUserMedia extends CApiService {
 			$sqlParts['where'][] = dbConditionInt('ug.usrgrpid', $options['usrgrpids']);
 			$sqlParts['where']['mug'] = 'm.userid=ug.userid';
 
-			if ($options['groupCount'] !== null) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['usrgrpid'] = 'ug.usrgrpid';
 			}
 		}
@@ -138,7 +140,7 @@ class CUserMedia extends CApiService {
 
 			$sqlParts['where'][] = dbConditionInt('m.mediatypeid', $options['mediatypeids']);
 
-			if ($options['groupCount'] !== null) {
+			if ($options['groupCount']) {
 				$sqlParts['group']['mediatypeid'] = 'm.mediatypeid';
 			}
 		}
@@ -163,8 +165,8 @@ class CUserMedia extends CApiService {
 		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 
 		while ($media = DBfetch($res)) {
-			if ($options['countOutput'] !== null) {
-				if ($options['groupCount'] !== null) {
+			if ($options['countOutput']) {
+				if ($options['groupCount']) {
 					$result[] = $media;
 				}
 				else {
@@ -176,12 +178,12 @@ class CUserMedia extends CApiService {
 			}
 		}
 
-		if ($options['countOutput'] !== null) {
+		if ($options['countOutput']) {
 			return $result;
 		}
 
 		// removing keys
-		if ($options['preservekeys'] === null) {
+		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
 
