@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,8 +26,15 @@ class CConfigFile {
 
 	const CONFIG_FILE_PATH = '/conf/zabbix.conf.php';
 
+	private static $supported_db_types = [
+		ZBX_DB_DB2 => true,
+		ZBX_DB_MYSQL => true,
+		ZBX_DB_ORACLE => true,
+		ZBX_DB_POSTGRESQL => true
+	];
+
 	public $configFile = null;
-	public $config = array();
+	public $config = [];
 	public $error = '';
 
 	private static function exception($error, $code = self::CONFIG_ERROR) {
@@ -58,27 +65,33 @@ class CConfigFile {
 		include($this->configFile);
 		ob_end_clean();
 
-		// config file in plain php is bad
-		$dbs = array(ZBX_DB_MYSQL, ZBX_DB_POSTGRESQL, ZBX_DB_ORACLE, ZBX_DB_DB2, ZBX_DB_SQLITE3);
 		if (!isset($DB['TYPE'])) {
 			self::exception('DB type is not set.');
 		}
-		elseif (isset($DB['TYPE']) && !in_array($DB['TYPE'], $dbs)) {
-			self::exception('DB type has wrong value. Possible values '.implode(', ', $dbs));
+
+		if (!array_key_exists($DB['TYPE'], self::$supported_db_types)) {
+			self::exception(
+				'Incorrect value "'.$DB['TYPE'].'" for DB type. Possible values '.
+				implode(', ', array_keys(self::$supported_db_types)).'.'
+			);
 		}
-		elseif (!isset($DB['DATABASE'])) {
+
+		$php_supported_db = array_keys(CFrontendSetup::getSupportedDatabases());
+
+		if (!in_array($DB['TYPE'], $php_supported_db)) {
+			self::exception('DB type "'.$DB['TYPE'].'" is not supported by current setup.'.
+				($php_supported_db ? ' Possible values '.implode(', ', $php_supported_db).'.' : '')
+			);
+		}
+
+		if (!isset($DB['DATABASE'])) {
 			self::exception('DB database is not set.');
 		}
 
 		$this->setDefaults();
 
-		if (isset($DB['TYPE'])) {
-			$this->config['DB']['TYPE'] = $DB['TYPE'];
-		}
-
-		if (isset($DB['DATABASE'])) {
-			$this->config['DB']['DATABASE'] = $DB['DATABASE'];
-		}
+		$this->config['DB']['TYPE'] = $DB['TYPE'];
+		$this->config['DB']['DATABASE'] = $DB['DATABASE'];
 
 		if (isset($DB['SERVER'])) {
 			$this->config['DB']['SERVER'] = $DB['SERVER'];
@@ -133,8 +146,17 @@ class CConfigFile {
 			$this->check();
 
 			if (!file_put_contents($this->configFile, $this->getString())) {
-				self::exception('Cannot write config file.');
+				if (file_exists($this->configFile)) {
+					if (file_get_contents($this->configFile) !== $this->getString()) {
+						self::exception(_('Unable to overwrite the existing configuration file.'));
+					}
+				}
+				else {
+					self::exception(_('Unable to create the configuration file.'));
+				}
 			}
+
+			return true;
 		}
 		catch (Exception $e) {
 			$this->error = $e->getMessage();
@@ -163,12 +185,11 @@ $ZBX_SERVER_PORT = \''.addcslashes($this->config['ZBX_SERVER_PORT'], "'\\").'\';
 $ZBX_SERVER_NAME = \''.addcslashes($this->config['ZBX_SERVER_NAME'], "'\\").'\';
 
 $IMAGE_FORMAT_DEFAULT = IMAGE_FORMAT_PNG;
-?>
 ';
 	}
 
 	protected function setDefaults() {
-		$this->config['DB'] = array(
+		$this->config['DB'] = [
 			'TYPE' => null,
 			'SERVER' => 'localhost',
 			'PORT' => '0',
@@ -176,22 +197,25 @@ $IMAGE_FORMAT_DEFAULT = IMAGE_FORMAT_PNG;
 			'USER' => '',
 			'PASSWORD' => '',
 			'SCHEMA' => ''
-		);
+		];
 		$this->config['ZBX_SERVER'] = 'localhost';
 		$this->config['ZBX_SERVER_PORT'] = '10051';
 		$this->config['ZBX_SERVER_NAME'] = '';
 	}
 
 	protected function check() {
-		$dbs = array(ZBX_DB_MYSQL, ZBX_DB_POSTGRESQL, ZBX_DB_ORACLE, ZBX_DB_DB2, ZBX_DB_SQLITE3);
-
 		if (!isset($this->config['DB']['TYPE'])) {
 			self::exception('DB type is not set.');
 		}
-		elseif (!in_array($this->config['DB']['TYPE'], $dbs)) {
-			self::exception('DB type has wrong value. Possible values '.implode(', ', $dbs));
+
+		if (!array_key_exists($this->config['DB']['TYPE'], self::$supported_db_types)) {
+			self::exception(
+				'Incorrect value "'.$this->config['DB']['TYPE'].'" for DB type. Possible values '.
+				implode(', ', array_keys(self::$supported_db_types)).'.'
+			);
 		}
-		elseif (!isset($this->config['DB']['DATABASE'])) {
+
+		if (!isset($this->config['DB']['DATABASE'])) {
 			self::exception('DB database is not set.');
 		}
 	}
