@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2015 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,13 +25,10 @@ class CMapImporter extends CImporter {
 	 * Import maps.
 	 *
 	 * @param array $maps
-	 *
-	 * @return void
 	 */
 	public function import(array $maps) {
 		$maps = zbx_toHash($maps, 'name');
 
-		$this->checkCircularMapReferences($maps);
 		$maps = $this->resolveMapElementReferences($maps);
 
 		/*
@@ -41,7 +38,7 @@ class CMapImporter extends CImporter {
 		 */
 		$mapsWithoutElements = $this->getMapsWithoutElements($maps);
 
-		$mapsToProcess = array('createMissing' => array(), 'updateExisting' => array());
+		$mapsToProcess = ['createMissing' => [], 'updateExisting' => []];
 
 		foreach ($mapsWithoutElements as $mapName => $mapWithoutElements) {
 			$mapId = $this->referencer->resolveMap($mapWithoutElements['name']);
@@ -72,16 +69,18 @@ class CMapImporter extends CImporter {
 		}
 
 		// Form an array of maps that need to be updated with elements and links, respecting the create/update options.
-		$mapsToUpdate = array();
+		$mapsToUpdate = [];
 		foreach ($mapsToProcess as $mapActionKey => $mapArray) {
 			if ($this->options['maps'][$mapActionKey] && $mapsToProcess[$mapActionKey]) {
 				foreach ($mapArray as $mapItem) {
-					$map = array(
+					$map = [
 						'sysmapid' => $maps[$mapItem['name']]['sysmapid'],
 						'name' => $mapItem['name'],
+						'shapes' => $maps[$mapItem['name']]['shapes'],
+						'lines' => $maps[$mapItem['name']]['lines'],
 						'selements' => $maps[$mapItem['name']]['selements'],
 						'links' => $maps[$mapItem['name']]['links']
-					);
+					];
 					$map = $this->resolveMapReferences($map);
 
 					// Remove the map name so API does not make an update query to the database.
@@ -96,76 +95,6 @@ class CMapImporter extends CImporter {
 		}
 	}
 
-	/**
-	 * Check if map elements have circular references.
-	 * Circular references can be only in map elements that represent another map.
-	 *
-	 * @throws Exception
-	 * @see checkCircularRecursive
-	 *
-	 * @param array $maps
-	 *
-	 * @return void
-	 */
-	protected function checkCircularMapReferences(array $maps) {
-		foreach ($maps as $mapName => $map) {
-			if (empty($map['selements'])) {
-				continue;
-			}
-
-			foreach ($map['selements'] as $selement) {
-				$checked = array($mapName);
-				if ($circMaps = $this->checkCircularRecursive($selement, $maps, $checked)) {
-					throw new Exception(_s('Circular reference in maps: %1$s.', implode(' - ', $circMaps)));
-				}
-			}
-		}
-	}
-
-	/**
-	 * Recursive function for searching for circular map references.
-	 * If circular reference exist it return array with map elements with circular reference.
-	 *
-	 * @param array $element map element to inspect on current recursive loop
-	 * @param array $maps    all maps where circular references should be searched
-	 * @param array $checked map names that already were processed,
-	 *                       should contain unique values if no circular references exist
-	 *
-	 * @return array|bool
-	 */
-	protected function checkCircularRecursive(array $element, array $maps, array $checked) {
-		// if element is not map element, recursive reference cannot happen
-		if ($element['elementtype'] != SYSMAP_ELEMENT_TYPE_MAP) {
-			return false;
-		}
-
-		$elementMapName = $element['element']['name'];
-
-		// if current element map name is already in list of checked map names,
-		// circular reference exists
-		if (in_array($elementMapName, $checked)) {
-			// to have nice result containing only maps that have circular reference,
-			// remove everything that was added before repeated map name
-			$checked = array_slice($checked, array_search($elementMapName, $checked));
-			// add repeated name to have nice loop like m1->m2->m3->m1
-			$checked[] = $elementMapName;
-			return $checked;
-		}
-		else {
-			$checked[] = $elementMapName;
-		}
-
-		// we need to find maps that reference the current element
-		// and if one has selements, check all of them recursively
-		if (!empty($maps[$elementMapName]['selements'])) {
-			foreach ($maps[$elementMapName]['selements'] as $selement) {
-				return $this->checkCircularRecursive($selement, $maps, $checked);
-			}
-		}
-
-		return false;
-	}
-
 	/**
 	 * Return maps without their elements.
 	 *
@@ -201,53 +130,65 @@ class CMapImporter extends CImporter {
 			foreach ($map['selements'] as &$selement) {
 				switch ($selement['elementtype']) {
 					case SYSMAP_ELEMENT_TYPE_MAP:
-						$selement['elementid'] = $this->referencer->resolveMap($selement['element']['name']);
-						if (!$selement['elementid']) {
+						$selement['elements'][0]['sysmapid'] = $this->referencer->resolveMap($selement['elements'][0]['name']);
+						if (!$selement['elements'][0]['sysmapid']) {
 							throw new Exception(_s('Cannot find map "%1$s" used in map "%2$s".',
-								$selement['element']['name'], $map['name']));
+								$selement['elements'][0]['name'], $map['name']));
 						}
+
+						unset($selement['elements'][0]['name']);
 						break;
 
 					case SYSMAP_ELEMENT_TYPE_HOST_GROUP:
-						$selement['elementid'] = $this->referencer->resolveGroup($selement['element']['name']);
-						if (!$selement['elementid']) {
+						$selement['elements'][0]['groupid'] = $this->referencer->resolveGroup($selement['elements'][0]['name']);
+						if (!$selement['elements'][0]['groupid']) {
 							throw new Exception(_s('Cannot find group "%1$s" used in map "%2$s".',
-								$selement['element']['name'], $map['name']));
+								$selement['elements'][0]['name'], $map['name']));
 						}
+
+						unset($selement['elements'][0]['name']);
 						break;
 
 					case SYSMAP_ELEMENT_TYPE_HOST:
-						$selement['elementid'] = $this->referencer->resolveHost($selement['element']['host']);
-						if (!$selement['elementid']) {
+						$selement['elements'][0]['hostid'] = $this->referencer->resolveHost($selement['elements'][0]['host']);
+						if (!$selement['elements'][0]['hostid']) {
 							throw new Exception(_s('Cannot find host "%1$s" used in map "%2$s".',
-								$selement['element']['host'], $map['name']));
+								$selement['elements'][0]['host'], $map['name']));
 						}
+
+						unset($selement['elements'][0]['host']);
 						break;
 
 					case SYSMAP_ELEMENT_TYPE_TRIGGER:
-						$el = $selement['element'];
-						$selement['elementid'] = $this->referencer->resolveTrigger($el['description'], $el['expression']);
+						foreach ($selement['elements'] as &$element) {
+							$element['triggerid'] = $this->referencer->resolveTrigger($element['description'],
+								$element['expression'], $element['recovery_expression']
+							);
 
-						if (!$selement['elementid']) {
-							throw new Exception(_s(
-								'Cannot find trigger "%1$s" used in map "%2$s".',
-								$selement['element']['description'],
-								$map['name']
-							));
+							if (!$element['triggerid']) {
+								throw new Exception(_s(
+									'Cannot find trigger "%1$s" used in map "%2$s".',
+									$element['description'],
+									$map['name']
+								));
+							}
+
+							unset($element['description'], $element['expression'], $element['recovery_expression']);
 						}
+						unset($element);
 						break;
 
 					case SYSMAP_ELEMENT_TYPE_IMAGE:
-						$selement['elementid'] = 0;
+						unset($selement['elements']);
 						break;
 				}
 
-				$icons = array(
+				$icons = [
 					'icon_off' => 'iconid_off',
 					'icon_on' => 'iconid_on',
 					'icon_disabled' => 'iconid_disabled',
 					'icon_maintenance' => 'iconid_maintenance',
-				);
+				];
 				foreach ($icons as $element => $field) {
 					if (!empty($selement[$element])) {
 						$image = getImageByIdent($selement[$element]);
@@ -271,7 +212,9 @@ class CMapImporter extends CImporter {
 
 				foreach ($link['linktriggers'] as &$linkTrigger) {
 					$trigger = $linkTrigger['trigger'];
-					$triggerId = $this->referencer->resolveTrigger($trigger['description'], $trigger['expression']);
+					$triggerId = $this->referencer->resolveTrigger($trigger['description'], $trigger['expression'],
+						$trigger['recovery_expression']
+					);
 
 					if (!$triggerId) {
 						throw new Exception(_s(
@@ -302,7 +245,7 @@ class CMapImporter extends CImporter {
 	 */
 	protected function resolveMapElementReferences(array $maps) {
 		foreach ($maps as &$map) {
-			if ($map['iconmap']) {
+			if (isset($map['iconmap']) && $map['iconmap']) {
 				$map['iconmapid'] = $this->referencer->resolveIconMap($map['iconmap']['name']);
 
 				if (!$map['iconmapid']) {
@@ -312,7 +255,7 @@ class CMapImporter extends CImporter {
 				}
 			}
 
-			if ($map['background']) {
+			if (isset($map['background']) && $map['background']) {
 				$image = getImageByIdent($map['background']);
 
 				if (!$image) {
