@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -40,7 +40,8 @@ class CScreenScreen extends CScreenBase {
 			'timestamp' => $this->timestamp,
 			'screen' => $screen,
 			'period' => $this->timeline['period'],
-			'stime' => $this->timeline['stimeNow'],
+			'stime' => $this->timeline['stime'],
+			'isNow' => $this->timeline['isNow'],
 			'profileIdx' => $this->profileIdx,
 			'updateProfile' => false
 		]);
