@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ class CUpdateDiscoveredValidator extends CValidator implements CPartialValidator
 	 *
 	 * @var array
 	 */
-	public $allowed = array();
+	public $allowed = [];
 
 	/**
 	 * Error message in case updating discovered objects is totally forbidden.
