@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -99,7 +99,8 @@ function item_type2str($type = null) {
 		ITEM_TYPE_TELNET => _('TELNET agent'),
 		ITEM_TYPE_JMX => _('JMX agent'),
 		ITEM_TYPE_CALCULATED => _('Calculated'),
-		ITEM_TYPE_HTTPTEST => _('Web monitoring')
+		ITEM_TYPE_HTTPTEST => _('Web monitoring'),
+		ITEM_TYPE_DEPENDENT => _('Dependent item')
 	];
 	if (is_null($type)) {
 		return $types;
@@ -135,24 +136,6 @@ function itemValueTypeString($valueType) {
 	return _('Unknown');
 }
 
-function item_data_type2str($type = null) {
-	$types = [
-		ITEM_DATA_TYPE_BOOLEAN => _('Boolean'),
-		ITEM_DATA_TYPE_OCTAL => _('Octal'),
-		ITEM_DATA_TYPE_DECIMAL => _('Decimal'),
-		ITEM_DATA_TYPE_HEXADECIMAL => _('Hexadecimal')
-	];
-	if (is_null($type)) {
-		return $types;
-	}
-	elseif (isset($types[$type])) {
-		return $types[$type];
-	}
-	else {
-		return _('Unknown');
-	}
-}
-
 function item_status2str($type = null) {
 	if (is_null($type)) {
 		return [ITEM_STATUS_ACTIVE => _('Enabled'), ITEM_STATUS_DISABLED => _('Disabled')];
@@ -224,6 +207,101 @@ function itemIndicatorStyle($status, $state = null) {
 	return ZBX_STYLE_RED;
 }
 
+/**
+ * Order items by keep history.
+ *
+ * @param array  $items
+ * @param string $items['history']
+ * @param string $sortorder
+ */
+function orderItemsByHistory(array &$items, $sortorder){
+	$simple_interval_parser = new CSimpleIntervalParser();
+
+	foreach ($items as &$item) {
+		$item['history_sort'] = ($simple_interval_parser->parse($item['history']) == CParser::PARSE_SUCCESS)
+			? timeUnitToSeconds($item['history'])
+			: $item['history'];
+	}
+	unset($item);
+
+	order_result($items, 'history_sort', $sortorder);
+
+	foreach ($items as &$item) {
+		unset($item['history_sort']);
+	}
+	unset($item);
+}
+
+/**
+ * Order items by keep trends.
+ *
+ * @param array  $items
+ * @param int    $items['value_type']
+ * @param string $items['trends']
+ * @param string $sortorder
+ */
+function orderItemsByTrends(array &$items, $sortorder){
+	$simple_interval_parser = new CSimpleIntervalParser();
+
+	foreach ($items as &$item) {
+		if (in_array($item['value_type'], [ITEM_VALUE_TYPE_STR, ITEM_VALUE_TYPE_LOG, ITEM_VALUE_TYPE_TEXT])) {
+			$item['trends_sort'] = '';
+		}
+		else {
+			$item['trends_sort'] = ($simple_interval_parser->parse($item['trends']) == CParser::PARSE_SUCCESS)
+				? timeUnitToSeconds($item['trends'])
+				: $item['trends'];
+		}
+	}
+	unset($item);
+
+	order_result($items, 'trends_sort', $sortorder);
+
+	foreach ($items as &$item) {
+		unset($item['trends_sort']);
+	}
+	unset($item);
+}
+
+/**
+ * Order items by update interval.
+ *
+ * @param array  $items
+ * @param int    $items['type']
+ * @param string $items['delay']
+ * @param string $sortorder
+ * @param array  $options
+ * @param bool   $options['usermacros']
+ * @param bool   $options['lldmacros']
+ */
+function orderItemsByDelay(array &$items, $sortorder, array $options){
+	$update_interval_parser = new CUpdateIntervalParser($options);
+
+	foreach ($items as &$item) {
+		if (in_array($item['type'], [ITEM_TYPE_TRAPPER, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT])) {
+			$item['delay_sort'] = '';
+		}
+		elseif ($update_interval_parser->parse($item['delay']) == CParser::PARSE_SUCCESS) {
+			$item['delay_sort'] = $update_interval_parser->getDelay();
+
+			if ($item['delay_sort'][0] !== '{') {
+				$item['delay_sort'] = timeUnitToSeconds($item['delay_sort']);
+			}
+		}
+		else {
+			$item['delay_sort'] = $item['delay'];
+		}
+	}
+	unset($item);
+
+	order_result($items, 'delay_sort', $sortorder);
+
+	foreach ($items as &$item) {
+		unset($item['delay_sort']);
+	}
+	unset($item);
+}
+
 /**
  * Orders items by both status and state. Items are sorted in the following order: enabled, disabled, not supported.
  *
@@ -304,25 +382,68 @@ function itemTypeInterface($type = null) {
 /**
  * Copies the given items to the given hosts or templates.
  *
- * @param array $src_itemids		Items which will be copied to $dst_hostids
- * @param array $dst_hostids		Hosts and templates to whom add items.
+ * @param array $src_itemids  Items which will be copied to $dst_hostids.
+ * @param array $dst_hostids  Hosts and templates to whom add items.
  *
  * @return bool
  */
 function copyItemsToHosts($src_itemids, $dst_hostids) {
 	$items = API::Item()->get([
-		'output' => [
-			'type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status', 'value_type',
-			'trapper_hosts', 'units', 'multiplier', 'delta', 'snmpv3_contextname', 'snmpv3_securityname',
-			'snmpv3_securitylevel', 'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol',
-			'snmpv3_privpassphrase', 'formula', 'logtimefmt', 'valuemapid', 'delay_flex', 'params', 'ipmi_sensor',
-			'data_type', 'authtype', 'username', 'password', 'publickey', 'privatekey', 'flags', 'port',
-			'description', 'inventory_link'
+		'output' => ['type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
+			'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname', 'snmpv3_securitylevel',
+			'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol', 'snmpv3_privpassphrase',
+			'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey',
+			'privatekey', 'flags', 'port', 'description', 'inventory_link', 'jmx_endpoint', 'master_itemid'
 		],
 		'selectApplications' => ['applicationid'],
-		'itemids' => $src_itemids
+		'selectPreprocessing' => ['type', 'params'],
+		'itemids' => $src_itemids,
+		'preservekeys' => true
 	]);
 
+	// Check if dependent items have master items in same selection. If not, those could be web items.
+	$master_itemids = [];
+
+	foreach ($items as $itemid => $item) {
+		if ($item['type'] == ITEM_TYPE_DEPENDENT && !array_key_exists($item['master_itemid'], $items)) {
+			$master_itemids[$item['master_itemid']] = true;
+		}
+	}
+
+	// Find same master items (that includes web items) on destination host.
+	$dst_master_items = [];
+
+	foreach (array_keys($master_itemids) as $master_itemid) {
+		$same_master_item = get_same_item_for_host(['itemid' => $master_itemid], $dst_hostids);
+
+		if ($same_master_item) {
+			$dst_master_items[$master_itemid] = $same_master_item;
+		}
+	}
+
+	$create_order = [];
+	$src_itemid_to_key = [];
+
+	// Calculate dependency level between items so that master items are created before dependent items.
+	foreach ($items as $itemid => $item) {
+		$dependency_level = 0;
+		$master_item = $item;
+		$src_itemid_to_key[$itemid] = $item['key_'];
+
+		while ($master_item['type'] == ITEM_TYPE_DEPENDENT) {
+			if (!array_key_exists($master_item['master_itemid'], $items)) {
+				break;
+			}
+
+			$master_item = $items[$master_item['master_itemid']];
+			++$dependency_level;
+		}
+
+		$create_order[$itemid] = $dependency_level;
+	}
+
+	asort($create_order);
+
 	$dstHosts = API::Host()->get([
 		'output' => ['hostid', 'host', 'status'],
 		'selectInterfaces' => ['interfaceid', 'type', 'main'],
@@ -334,12 +455,36 @@ function copyItemsToHosts($src_itemids, $dst_hostids) {
 
 	foreach ($dstHosts as $dstHost) {
 		$interfaceids = [];
+
 		foreach ($dstHost['interfaces'] as $interface) {
 			if ($interface['main'] == 1) {
 				$interfaceids[$interface['type']] = $interface['interfaceid'];
 			}
 		}
-		foreach ($items as &$item) {
+
+		$itemkey_to_id = [];
+		$create_items = [];
+		$current_dependency = reset($create_order);
+
+		foreach ($create_order as $itemid => $dependency_level) {
+			if ($current_dependency != $dependency_level) {
+				$current_dependency = $dependency_level;
+				$created_itemids = API::Item()->create($create_items);
+
+				if (!$created_itemids) {
+					return false;
+				}
+				$created_itemids = $created_itemids['itemids'];
+
+				foreach ($create_items as $index => $created_item) {
+					$itemkey_to_id[$created_item['key_']] = $created_itemids[$index];
+				}
+
+				$create_items = [];
+			}
+
+			$item = $items[$itemid];
+
 			if ($dstHost['status'] != HOST_STATUS_TEMPLATE) {
 				$type = itemTypeInterface($item['type']);
 
@@ -367,10 +512,42 @@ function copyItemsToHosts($src_itemids, $dst_hostids) {
 				zbx_objectValues($item['applications'], 'applicationid'),
 				$dstHost['hostid']
 			);
-		}
-		unset($item);
 
-		if (!API::Item()->create($items)) {
+			if ($item['type'] == ITEM_TYPE_DEPENDENT) {
+				if (array_key_exists($item['master_itemid'], $items)) {
+					$src_item_key = $src_itemid_to_key[$item['master_itemid']];
+					$item['master_itemid'] = $itemkey_to_id[$src_item_key];
+				}
+				else {
+					$item_found = false;
+
+					if (array_key_exists($item['master_itemid'], $dst_master_items)) {
+						foreach ($dst_master_items[$item['master_itemid']] as $dst_master_item) {
+							if ($dst_master_item['hostid'] == $dstHost['hostid']) {
+								// A matching item on destination host has been found.
+
+								$item['master_itemid'] = $dst_master_item['itemid'];
+								$item_found = true;
+							}
+						}
+					}
+
+					// Master item does not exist on destination host or has not been selected for copying.
+					if (!$item_found) {
+						error(_s('Item "%1$s" has master item and cannot be copied.', $item['name']));
+
+						return false;
+					}
+				}
+			}
+			else {
+				unset($item['master_itemid']);
+			}
+
+			$create_items[] = $item;
+		}
+
+		if ($create_items && !API::Item()->create($create_items)) {
 			return false;
 		}
 	}
@@ -380,18 +557,19 @@ function copyItemsToHosts($src_itemids, $dst_hostids) {
 
 function copyItems($srcHostId, $dstHostId) {
 	$srcItems = API::Item()->get([
-		'hostids' => $srcHostId,
-		'output' => [
-			'type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status', 'value_type',
-			'trapper_hosts', 'units', 'multiplier', 'delta', 'snmpv3_contextname', 'snmpv3_securityname',
-			'snmpv3_securitylevel', 'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol',
-			'snmpv3_privpassphrase', 'formula', 'logtimefmt', 'valuemapid', 'delay_flex', 'params', 'ipmi_sensor',
-			'data_type', 'authtype', 'username', 'password', 'publickey', 'privatekey', 'flags', 'port',
-			'description', 'inventory_link'
+		'output' => ['type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
+			'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname', 'snmpv3_securitylevel',
+			'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol', 'snmpv3_privpassphrase',
+			'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey',
+			'privatekey', 'flags', 'port', 'description', 'inventory_link', 'jmx_endpoint', 'master_itemid',
+			'templateid'
 		],
-		'inherited' => false,
+		'selectApplications' => ['applicationid'],
+		'selectPreprocessing' => ['type', 'params'],
+		'hostids' => $srcHostId,
+		'webitems' => true,
 		'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
-		'selectApplications' => ['applicationid']
+		'preservekeys' => true
 	]);
 	$dstHosts = API::Host()->get([
 		'output' => ['hostid', 'host', 'status'],
@@ -403,10 +581,63 @@ function copyItems($srcHostId, $dstHostId) {
 	]);
 	$dstHost = reset($dstHosts);
 
-	foreach ($srcItems as &$srcItem) {
+	$create_order = [];
+	$src_itemid_to_key = [];
+	foreach ($srcItems as $itemid => $item) {
+		$dependency_level = 0;
+		$master_item = $item;
+		$src_itemid_to_key[$itemid] = $item['key_'];
+
+		while ($master_item['type'] == ITEM_TYPE_DEPENDENT) {
+			$master_item = $srcItems[$master_item['master_itemid']];
+			++$dependency_level;
+		}
+
+		$create_order[$itemid] = $dependency_level;
+	}
+	asort($create_order);
+
+	$itemkey_to_id = [];
+	$create_items = [];
+	$current_dependency = reset($create_order);
+
+	foreach ($create_order as $itemid => $dependency_level) {
+		$srcItem = $srcItems[$itemid];
+
+		// Skip creating web items. Those were created before.
+		if ($srcItem['type'] == ITEM_TYPE_HTTPTEST) {
+			continue;
+		}
+
+		if ($current_dependency != $dependency_level && $create_items) {
+			$current_dependency = $dependency_level;
+			$created_itemids = API::Item()->create($create_items);
+
+			if (!$created_itemids) {
+				return false;
+			}
+			$created_itemids = $created_itemids['itemids'];
+
+			foreach ($create_items as $index => $created_item) {
+				$itemkey_to_id[$created_item['key_']] = $created_itemids[$index];
+			}
+
+			$create_items = [];
+		}
+
+		if ($srcItem['templateid']) {
+			$srcItem = get_same_item_for_host($srcItem, $dstHost['hostid']);
+
+			if (!$srcItem) {
+				return false;
+			}
+			$itemkey_to_id[$srcItem['key_']] = $srcItem['itemid'];
+			continue;
+		}
+
 		if ($dstHost['status'] != HOST_STATUS_TEMPLATE) {
 			// find a matching interface
-			$interface = CItem::findInterfaceForItem($srcItem, $dstHost['interfaces']);
+			$interface = CItem::findInterfaceForItem($srcItem['type'], $dstHost['interfaces']);
 			if ($interface) {
 				$srcItem['interfaceid'] = $interface['interfaceid'];
 			}
@@ -419,10 +650,34 @@ function copyItems($srcHostId, $dstHostId) {
 		unset($srcItem['templateid']);
 		$srcItem['hostid'] = $dstHostId;
 		$srcItem['applications'] = get_same_applications_for_host(zbx_objectValues($srcItem['applications'], 'applicationid'), $dstHostId);
-	}
-	unset($srcItem);
 
-	return API::Item()->create($srcItems);
+		if (!$srcItem['preprocessing']) {
+			unset($srcItem['preprocessing']);
+		}
+
+		if ($srcItem['type'] == ITEM_TYPE_DEPENDENT) {
+			if ($srcItems[$srcItem['master_itemid']]['type'] == ITEM_TYPE_HTTPTEST) {
+				// Web items are outside the scope and are created before regular items.
+				$web_item = get_same_item_for_host($srcItems[$srcItem['master_itemid']], $dstHost['hostid']);
+				$srcItem['master_itemid'] = $web_item['itemid'];
+			}
+			else {
+				$src_item_key = $src_itemid_to_key[$srcItem['master_itemid']];
+				$srcItem['master_itemid'] = $itemkey_to_id[$src_item_key];
+			}
+		}
+		else {
+			unset($srcItem['master_itemid']);
+		}
+
+		$create_items[] = $srcItem;
+	}
+
+	if ($create_items && !API::Item()->create($create_items)) {
+		return false;
+	}
+
+	return true;
 }
 
 /**
@@ -466,11 +721,11 @@ function get_item_by_itemid($itemid) {
 function get_item_by_itemid_limited($itemid) {
 	$row = DBfetch(DBselect(
 		'SELECT i.itemid,i.interfaceid,i.name,i.key_,i.hostid,i.delay,i.history,i.status,i.type,i.lifetime,'.
-			'i.snmp_community,i.snmp_oid,i.value_type,i.data_type,i.trapper_hosts,i.port,i.units,i.multiplier,'.
-			'i.delta,i.snmpv3_contextname,i.snmpv3_securityname,i.snmpv3_securitylevel,i.snmpv3_authprotocol,'.
-			'i.snmpv3_authpassphrase,i.snmpv3_privprotocol,i.snmpv3_privpassphrase,i.formula,i.trends,i.logtimefmt,'.
-			'i.valuemapid,i.delay_flex,i.params,i.ipmi_sensor,i.templateid,i.authtype,i.username,i.password,'.
-			'i.publickey,i.privatekey,i.flags,i.description,i.inventory_link'.
+			'i.snmp_community,i.snmp_oid,i.value_type,i.trapper_hosts,i.port,i.units,i.snmpv3_contextname,'.
+			'i.snmpv3_securityname,i.snmpv3_securitylevel,i.snmpv3_authprotocol,i.snmpv3_authpassphrase,'.
+			'i.snmpv3_privprotocol,i.snmpv3_privpassphrase,i.trends,i.logtimefmt,i.valuemapid,i.params,i.ipmi_sensor,'.
+			'i.templateid,i.authtype,i.username,i.password,i.publickey,i.privatekey,i.flags,i.description,'.
+			'i.inventory_link'.
 		' FROM items i'.
 		' WHERE i.itemid='.zbx_dbstr($itemid)));
 	if ($row) {
@@ -567,50 +822,82 @@ function get_realrule_by_itemid_and_hostid($itemid, $hostid) {
 /**
  * Retrieve overview table object for items.
  *
- * @param array  		$hostIds
- * @param array|null	$applicationIds		IDs of applications to filter items by
- * @param int    		$viewMode
+ * @param array|null $groupids
+ * @param string     $application  IDs of applications to filter items by
+ * @param int        $viewMode
  *
  * @return CTableInfo
  */
-function getItemsDataOverview($hostIds, array $applicationIds = null, $viewMode) {
-	$sqlFrom = '';
-	$sqlWhere = '';
-
-	if ($applicationIds !== null) {
-		$sqlFrom = 'items_applications ia,';
-		$sqlWhere = ' AND i.itemid=ia.itemid AND '.dbConditionInt('ia.applicationid', $applicationIds);
+function getItemsDataOverview(array $groupids, $application, $viewMode) {
+	// application filter
+	if ($application !== '') {
+		$applicationids = array_keys(API::Application()->get([
+			'output' => [],
+			'groupids' => $groupids ? $groupids : null,
+			'search' => ['name' => $application],
+			'preservekeys' => true
+		]));
+		$groupids = [];
+	}
+	else {
+		$applicationids = null;
 	}
 
-	$dbItems = DBfetchArray(DBselect(
-		'SELECT DISTINCT h.hostid,h.name AS hostname,i.itemid,i.key_,i.value_type,i.units,'.
-			'i.name,t.priority,i.valuemapid,t.value AS tr_value,t.triggerid'.
-		' FROM hosts h,'.$sqlFrom.'items i'.
-			' LEFT JOIN functions f ON f.itemid=i.itemid'.
-			' LEFT JOIN triggers t ON t.triggerid=f.triggerid AND t.status='.TRIGGER_STATUS_ENABLED.
-		' WHERE '.dbConditionInt('h.hostid', $hostIds).
-			' AND h.status='.HOST_STATUS_MONITORED.
-			' AND h.hostid=i.hostid'.
-			' AND i.status='.ITEM_STATUS_ACTIVE.
-			' AND '.dbConditionInt('i.flags', [ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_CREATED]).
-				$sqlWhere
-	));
+	$db_items = API::Item()->get([
+		'output' => ['itemid', 'hostid', 'key_', 'name', 'value_type', 'units', 'valuemapid'],
+		'selectHosts' => ['name'],
+		'groupids' => $groupids ? $groupids : null,
+		'applicationids' => $applicationids,
+		'monitored' => true,
+		'webitems' => true,
+		'preservekeys' => true
+	]);
 
-	$dbItems = CMacrosResolverHelper::resolveItemNames($dbItems);
+	$db_triggers = API::Trigger()->get([
+		'output' => ['triggerid', 'priority', 'value'],
+		'selectItems' => ['itemid'],
+		'groupids' => $groupids ? $groupids : null,
+		'applicationids' => $applicationids,
+		'monitored' => true
+	]);
 
-	CArrayHelper::sort($dbItems, [
+	foreach ($db_triggers as $db_trigger) {
+		foreach ($db_trigger['items'] as $item) {
+			if (array_key_exists($item['itemid'], $db_items)) {
+				$db_item = &$db_items[$item['itemid']];
+
+				// a little tricky check for attempt to overwrite active trigger (value=1) with
+				// inactive or active trigger with lower priority.
+				if (!array_key_exists('triggerid', $db_item)
+						|| ($db_item['value'] == TRIGGER_VALUE_FALSE && $db_trigger['value'] == TRIGGER_VALUE_TRUE)
+						|| (($db_item['value'] == TRIGGER_VALUE_FALSE || $db_trigger['value'] == TRIGGER_VALUE_TRUE)
+							&& $db_item['priority'] < $db_trigger['priority'])) {
+					$db_item['triggerid'] = $db_trigger['triggerid'];
+					$db_item['priority'] = $db_trigger['priority'];
+					$db_item['value'] = $db_trigger['value'];
+				}
+
+				unset($db_item);
+			}
+		}
+	}
+
+	$db_items = CMacrosResolverHelper::resolveItemNames($db_items);
+
+	CArrayHelper::sort($db_items, [
 		['field' => 'name_expanded', 'order' => ZBX_SORT_UP],
 		['field' => 'itemid', 'order' => ZBX_SORT_UP]
 	]);
 
 	// fetch latest values
-	$history = Manager::History()->getLast(zbx_toHash($dbItems, 'itemid'), 1, ZBX_HISTORY_PERIOD);
+	$history = Manager::History()->getLastValues(zbx_toHash($db_items, 'itemid'), 1, ZBX_HISTORY_PERIOD);
 
 	// fetch data for the host JS menu
 	$hosts = API::Host()->get([
 		'output' => ['name', 'hostid', 'status'],
 		'monitored_hosts' => true,
-		'hostids' => $hostIds,
+		'groupids' => $groupids ? $groupids : null,
+		'applicationids' => $applicationids,
 		'with_monitored_items' => true,
 		'preservekeys' => true,
 		'selectGraphs' => API_OUTPUT_COUNT,
@@ -620,10 +907,10 @@ function getItemsDataOverview($hostIds, array $applicationIds = null, $viewMode)
 	$items = [];
 	$item_counter = [];
 	$host_items = [];
-	foreach ($dbItems as $dbItem) {
-		$item_name = $dbItem['name_expanded'];
-		$host_name = $dbItem['hostname'];
-		$hostNames[$dbItem['hostid']] = $host_name;
+	foreach ($db_items as $db_item) {
+		$item_name = $db_item['name_expanded'];
+		$host_name = $db_item['hosts'][0]['name'];
+		$hostNames[$db_item['hostid']] = $host_name;
 
 		if (!array_key_exists($host_name, $item_counter)) {
 			$item_counter[$host_name] = [];
@@ -637,34 +924,42 @@ function getItemsDataOverview($hostIds, array $applicationIds = null, $viewMode)
 			$host_items[$item_name][$host_name] = [];
 		}
 
-		// a little tricky check for attempt to overwrite active trigger (value=1) with
-		// inactive or active trigger with lower priority.
-		if (!array_key_exists($dbItem['itemid'], $host_items[$item_name][$host_name])
-			|| (($host_items[$item_name][$host_name][$dbItem['itemid']]['tr_value'] == TRIGGER_VALUE_FALSE && $dbItem['tr_value'] == TRIGGER_VALUE_TRUE)
-				|| (($host_items[$item_name][$host_name][$dbItem['itemid']]['tr_value'] == TRIGGER_VALUE_FALSE || $dbItem['tr_value'] == TRIGGER_VALUE_TRUE)
-					&& $dbItem['priority'] > $host_items[$item_name][$host_name][$dbItem['itemid']]['severity']))) {
-
-			if (array_key_exists($dbItem['itemid'], $host_items[$item_name][$host_name])) {
-				$item_place = $host_items[$item_name][$host_name][$dbItem['itemid']]['item_place'];
+		if (!array_key_exists($db_item['itemid'], $host_items[$item_name][$host_name])) {
+			if (array_key_exists($db_item['itemid'], $host_items[$item_name][$host_name])) {
+				$item_place = $host_items[$item_name][$host_name][$db_item['itemid']]['item_place'];
 			}
 			else {
 				$item_place = $item_counter[$host_name][$item_name];
 				$item_counter[$host_name][$item_name]++;
 			}
 
-			$items[$item_name][$item_place][$host_name] = [
-				'itemid' => $dbItem['itemid'],
-				'value_type' => $dbItem['value_type'],
-				'value' => isset($history[$dbItem['itemid']]) ? $history[$dbItem['itemid']][0]['value'] : null,
-				'units' => $dbItem['units'],
-				'valuemapid' => $dbItem['valuemapid'],
-				'severity' => $dbItem['priority'],
-				'tr_value' => $dbItem['tr_value'],
-				'triggerid' => $dbItem['triggerid'],
+			$item = [
+				'itemid' => $db_item['itemid'],
+				'value_type' => $db_item['value_type'],
+				'value' => isset($history[$db_item['itemid']]) ? $history[$db_item['itemid']][0]['value'] : null,
+				'units' => $db_item['units'],
+				'valuemapid' => $db_item['valuemapid'],
 				'item_place' => $item_place
 			];
 
-			$host_items[$item_name][$host_name][$dbItem['itemid']] = $items[$item_name][$item_place][$host_name];
+			if (array_key_exists('triggerid', $db_item)) {
+				$item += [
+					'triggerid' => $db_item['triggerid'],
+					'severity' => $db_item['priority'],
+					'tr_value' => $db_item['value']
+				];
+			}
+			else {
+				$item += [
+					'triggerid' => null,
+					'severity' => null,
+					'tr_value' => null
+				];
+			}
+
+			$items[$item_name][$item_place][$host_name] = $item;
+
+			$host_items[$item_name][$host_name][$db_item['itemid']] = $items[$item_name][$item_place][$host_name];
 		}
 	}
 
@@ -738,10 +1033,14 @@ function getItemDataOverviewCells($tableRow, $ithosts, $hostName) {
 			// Display event acknowledgement.
 			$config = select_config();
 			if ($config['event_ack_enable']) {
-				$ack = get_last_event_by_triggerid($item['triggerid']);
-				$ack = ($ack['acknowledged'] == 1)
-					? [SPACE, (new CSpan())->addClass(ZBX_STYLE_ICON_ACKN)]
-					: null;
+				$ack = getTriggerLastProblems([$item['triggerid']], ['acknowledged']);
+
+				if ($ack) {
+					$ack = reset($ack);
+					$ack = ($ack['acknowledged'] == 1)
+						? [SPACE, (new CSpan())->addClass(ZBX_STYLE_ICON_ACKN)]
+						: null;
+				}
 			}
 		}
 
@@ -818,25 +1117,6 @@ function get_applications_by_itemid($itemids, $field = 'applicationid') {
 	return $result;
 }
 
-/**
- * Clear item history and trends by provided item IDs.
- *
- * @param array $itemIds
- *
- * @return bool
- */
-function deleteHistoryByItemIds(array $itemIds) {
-	$result = DBexecute('DELETE FROM trends WHERE '.dbConditionInt('itemid', $itemIds));
-	$result = ($result && DBexecute('DELETE FROM trends_uint WHERE '.dbConditionInt('itemid', $itemIds)));
-	$result = ($result && DBexecute('DELETE FROM history_text WHERE '.dbConditionInt('itemid', $itemIds)));
-	$result = ($result && DBexecute('DELETE FROM history_log WHERE '.dbConditionInt('itemid', $itemIds)));
-	$result = ($result && DBexecute('DELETE FROM history_uint WHERE '.dbConditionInt('itemid', $itemIds)));
-	$result = ($result && DBexecute('DELETE FROM history_str WHERE '.dbConditionInt('itemid', $itemIds)));
-	$result = ($result && DBexecute('DELETE FROM history WHERE '.dbConditionInt('itemid', $itemIds)));
-
-	return $result;
-}
-
 /**
  * Format history value.
  * First format the value according to the configuration of the item. Then apply the value mapping to the formatted (!)
@@ -916,22 +1196,16 @@ function getItemFunctionalValue($item, $function, $parameter) {
 	}
 
 	// allowed item types for min, max and avg function
-	$historyTables = [ITEM_VALUE_TYPE_FLOAT => 'history', ITEM_VALUE_TYPE_UINT64 => 'history_uint'];
+	$history_tables = [ITEM_VALUE_TYPE_FLOAT => 'history', ITEM_VALUE_TYPE_UINT64 => 'history_uint'];
 
-	if (!isset($historyTables[$item['value_type']])) {
+	if (!array_key_exists($item['value_type'], $history_tables)) {
 		return UNRESOLVED_MACRO_STRING;
 	}
 	else {
-		// search for item function data in DB corresponding history table
-		$result = DBselect(
-			'SELECT '.$function.'(value) AS value'.
-			' FROM '.$historyTables[$item['value_type']].
-			' WHERE clock>'.(time() - $parameter).
-			' AND itemid='.zbx_dbstr($item['itemid']).
-			' HAVING COUNT(*)>0' // necessary because DBselect() return 0 if empty data set, for graph templates
-		);
-		if ($row = DBfetch($result)) {
-			return convert_units(['value' => $row['value'], 'units' => $item['units']]);
+		$result = Manager::History()->getAggregatedValue($item, $function, (time() - $parameter));
+
+		if ($result !== null) {
+			return convert_units(['value' => $result, 'units' => $item['units']]);
 		}
 		// no data in history
 		else {
@@ -940,81 +1214,6 @@ function getItemFunctionalValue($item, $function, $parameter) {
 	}
 }
 
-/**
- * Returns the history value of the item at the given time. If no value exists at the given time, the function
- * will return the previous value.
- *
- * The $db_item parameter must have the value_type and itemid properties set.
- *
- * @param array $db_item
- * @param int $clock
- * @param int $ns
- *
- * @return string
- */
-function item_get_history($db_item, $clock, $ns) {
-	$value = null;
-
-	$table = CHistoryManager::getTableName($db_item['value_type']);
-
-	$sql = 'SELECT value'.
-			' FROM '.$table.
-			' WHERE itemid='.zbx_dbstr($db_item['itemid']).
-				' AND clock='.zbx_dbstr($clock).
-				' AND ns='.zbx_dbstr($ns);
-
-	if ($row = DBfetch(DBselect($sql, 1))) {
-		$value = $row['value'];
-	}
-	if ($value !== null) {
-		return $value;
-	}
-
-	$max_clock = 0;
-
-	$sql = 'SELECT DISTINCT clock'.
-			' FROM '.$table.
-			' WHERE itemid='.zbx_dbstr($db_item['itemid']).
-				' AND clock='.zbx_dbstr($clock).
-				' AND ns<'.zbx_dbstr($ns);
-	if (null != ($row = DBfetch(DBselect($sql)))) {
-		$max_clock = $row['clock'];
-	}
-	if ($max_clock == 0) {
-		$sql = 'SELECT MAX(clock) AS clock'.
-				' FROM '.$table.
-				' WHERE itemid='.zbx_dbstr($db_item['itemid']).
-					' AND clock<'.zbx_dbstr($clock);
-		if (null != ($row = DBfetch(DBselect($sql)))) {
-			$max_clock = $row['clock'];
-		}
-	}
-	if ($max_clock == 0) {
-		return $value;
-	}
-
-	if ($clock == $max_clock) {
-		$sql = 'SELECT value'.
-				' FROM '.$table.
-				' WHERE itemid='.zbx_dbstr($db_item['itemid']).
-					' AND clock='.zbx_dbstr($clock).
-					' AND ns<'.zbx_dbstr($ns);
-	}
-	else {
-		$sql = 'SELECT value'.
-				' FROM '.$table.
-				' WHERE itemid='.zbx_dbstr($db_item['itemid']).
-					' AND clock='.zbx_dbstr($max_clock).
-				' ORDER BY itemid,clock desc,ns desc';
-	}
-
-	if (null != ($row = DBfetch(DBselect($sql, 1)))) {
-		$value = $row['value'];
-	}
-
-	return $value;
-}
-
 /**
  * Check if current time is within the given period.
  *
@@ -1051,6 +1250,8 @@ function checkTimePeriod($period, $now) {
  * @return string
  */
 function getItemDelay($delay, array $flexible_intervals) {
+	$delay = timeUnitToSeconds($delay);
+
 	if ($delay != 0 || !$flexible_intervals) {
 		return $delay;
 	}
@@ -1059,7 +1260,7 @@ function getItemDelay($delay, array $flexible_intervals) {
 
 	foreach ($flexible_intervals as $flexible_interval) {
 		$flexible_interval_parts = explode('/', $flexible_interval);
-		$flexible_delay = (int) $flexible_interval_parts[0];
+		$flexible_delay = timeUnitToSeconds($flexible_interval_parts[0]);
 
 		$min_delay = min($min_delay, $flexible_delay);
 	}
@@ -1295,6 +1496,97 @@ function getParamFieldLabelByType($itemType) {
 	}
 }
 
+/**
+ * Get either one or all item preprocessing types. If grouped set to true, returns group labels. Returns empty string if
+ * no specific type is found.
+ *
+ * Usage examples:
+ *    - get_preprocessing_types()              Returns array as defined.
+ *    - get_preprocessing_types(4)             Returns string: 'Trim'.
+ *    - get_preprocessing_types(<wrong type>)  Returns an empty string: ''.
+ *    - get_preprocessing_types(null, false)   Returns subarrays in one array maintaining index:
+ *                                               [5] => Regular expression
+ *                                               [4] => Trim
+ *                                               [2] => Right trim
+ *                                               [3] => Left trim
+ *                                               [11] => XML XPath
+ *                                               [12] => JSON Path
+ *                                               [1] => Custom multiplier
+ *                                               [9] => Simple change
+ *                                               [10] => Speed per second
+ *                                               [6] => Boolean to decimal
+ *                                               [7] => Octal to decimal
+ *                                               [8] => Hexadecimal to decimal
+ *
+ * @param int  $type     Item preprocessing type.
+ * @param bool $grouped  Group label flag.
+ *
+ * @return mixed
+ */
+function get_preprocessing_types($type = null, $grouped = true) {
+	$groups = [
+		[
+			'label' => _('Text'),
+			'types' => [
+				ZBX_PREPROC_REGSUB => _('Regular expression'),
+				ZBX_PREPROC_TRIM => _('Trim'),
+				ZBX_PREPROC_RTRIM => _('Right trim'),
+				ZBX_PREPROC_LTRIM => _('Left trim')
+			]
+		],
+		[
+			'label' => _('Structured data'),
+			'types' => [
+				ZBX_PREPROC_XPATH => _('XML XPath'),
+				ZBX_PREPROC_JSONPATH => _('JSON Path')
+			]
+		],
+		[
+			'label' => _('Arithmetic'),
+			'types' => [
+				ZBX_PREPROC_MULTIPLIER => _('Custom multiplier')
+			]
+		],
+		[
+			'label' => _x('Change', 'noun'),
+			'types' => [
+				ZBX_PREPROC_DELTA_VALUE => _('Simple change'),
+				ZBX_PREPROC_DELTA_SPEED => _('Change per second')
+			]
+		],
+		[
+			'label' => _('Numeral systems'),
+			'types' => [
+				ZBX_PREPROC_BOOL2DEC => _('Boolean to decimal'),
+				ZBX_PREPROC_OCT2DEC => _('Octal to decimal'),
+				ZBX_PREPROC_HEX2DEC => _('Hexadecimal to decimal')
+			]
+		]
+	];
+
+	if ($type !== null) {
+		foreach ($groups as $group) {
+			if (array_key_exists($type, $group['types'])) {
+				return $group['types'][$type];
+			}
+		}
+
+		return '';
+	}
+	elseif ($grouped) {
+		return $groups;
+	}
+	else {
+		$types = [];
+
+		foreach ($groups as $group) {
+			$types += $group['types'];
+		}
+
+		return $types;
+	}
+}
+
 /*
  * Quoting $param if it contain special characters.
  *
@@ -1312,3 +1604,67 @@ function quoteItemKeyParam($param, $forced = false) {
 
 	return '"'.str_replace('"', '\\"', $param).'"';
 }
+
+/**
+ * Expands item name and for dependent item master item name.
+ *
+ * @param array  $items        Array of items.
+ * @param string $data_source  'items' or 'itemprototypes'.
+ *
+ * @return array
+ */
+function expandItemNamesWithMasterItems($items, $data_source) {
+	$items = CMacrosResolverHelper::resolveItemNames($items);
+	$itemids = [];
+	$master_itemids = [];
+
+	foreach ($items as $item_index => $item) {
+		if ($item['type'] == ITEM_TYPE_DEPENDENT) {
+			$master_itemids[$item['master_itemid']] = true;
+		}
+		$itemids[$item_index] = $item['itemid'];
+	}
+	$master_itemids = array_diff(array_keys($master_itemids), $itemids);
+
+	if ($master_itemids) {
+		if ($data_source === 'items') {
+			$master_items = API::Item()->get([
+				'output' => ['itemid', 'type', 'hostid', 'name', 'key_'],
+				'itemids' => $master_itemids,
+				'webitems' => true,
+				'editable' => true,
+				'preservekeys' => true
+			]);
+		}
+		elseif ($data_source === 'itemprototypes') {
+			$master_items = API::ItemPrototype()->get([
+				'output' => ['itemid', 'type', 'hostid', 'name', 'key_'],
+				'itemids' => $master_itemids,
+				'editable' => true,
+				'preservekeys' => true
+			]);
+		}
+
+		$master_items = CMacrosResolverHelper::resolveItemNames($master_items);
+	}
+
+	foreach ($items as &$item) {
+		if ($item['type'] == ITEM_TYPE_DEPENDENT) {
+			$master_itemid = $item['master_itemid'];
+			$items_index = array_search($master_itemid, $itemids);
+
+			$item['master_item'] = [
+				'itemid' => $master_itemid,
+				'name_expanded' => ($items_index === false)
+					? $master_items[$master_itemid]['name_expanded']
+					: $items[$items_index]['name_expanded'],
+				'type' => ($items_index === false)
+					? $master_items[$master_itemid]['type']
+					: $items[$items_index]['type'],
+			];
+		}
+	}
+	unset($item);
+
+	return $items;
+}
