@@ -62,7 +62,7 @@ function getLocales() {
 		'ka_GE' => ['name' => _('Georgian (ka_GE)'),	'display' => false],
 		'de_DE' => ['name' => _('German (de_DE)'),	'display' => false],
 		'el_GR' => ['name' => _('Greek (el_GR)'),	'display' => false],
-		'he_IL' => ['name' => _('Hebrew (he_IL)'),	'display' => false],
+		'he_IL' => ['name' => _('Hebrew (he_IL)'),	'display' => true],
 		'hu_HU' => ['name' => _('Hungarian (hu_HU)'),	'display' => false],
 		'id_ID' => ['name' => _('Indonesian (id_ID)'),	'display' => false],
 		'it_IT' => ['name' => _('Italian (it_IT)'),	'display' => true],
@@ -70,6 +70,7 @@ function getLocales() {
 		'ja_JP' => ['name' => _('Japanese (ja_JP)'),	'display' => true],
 		'lv_LV' => ['name' => _('Latvian (lv_LV)'),	'display' => false],
 		'lt_LT' => ['name' => _('Lithuanian (lt_LT)'),	'display' => false],
+		'nb_NO' => ['name' => _('Norwegian (nb_NO)'),	'display' => true],
 		'fa_IR' => ['name' => _('Persian (fa_IR)'),	'display' => false],
 		'pl_PL' => ['name' => _('Polish (pl_PL)'),	'display' => true],
 		'pt_BR' => ['name' => _('Portuguese (pt_BR)'),	'display' => true],
@@ -86,18 +87,18 @@ function getLocales() {
 }
 
 /**
- * Return an array of locale name variants based on language.
+ * Returns an array of locale name variants based on language.
  *
- * @param string $language in format 'ru_RU', 'en_EN' and so on
- * @return array a list of possible locale names
+ * @param string $language Language in format 'ru_RU', 'en_EN' and so on.
+ *
+ * @return array A list of possible locale names.
  */
 function zbx_locale_variants($language) {
 	if (strtolower(substr(PHP_OS, 0, 3)) === 'win') {
 		return zbx_locale_variants_win($language);
 	}
-	else {
-		return zbx_locale_variants_unix($language);
-	}
+
+	return zbx_locale_variants_unix($language);
 }
 
 function zbx_locale_variants_unix($language) {
@@ -137,19 +138,21 @@ function zbx_locale_variants_unix($language) {
 		'.koi8r',
 		'.tcvn'
 	];
+
 	$result = [];
 	foreach ($postfixes as $postfix) {
 		$result[] = $language.$postfix;
 	}
+
 	return $result;
 }
 
 function zbx_locale_variants_win($language) {
-	// windows locales are written like language[_country[.charset]]
-	// for a list of supported languages see:
+	// Windows locales are written like language[_country[.charset]].
+	// For a list of supported languages see:
 	// http://msdn.microsoft.com/en-us/library/39cwe7zf(vs.71).aspx
 	// http://docs.moodle.org/dev/Table_of_locales#Table
-	$winLanguageName = [
+	$win_language_names = [
 		'en_gb' => ['English_United Kingdom.1252', 'english-uk'],
 		'en_us' => ['English_United States.1252', 'english-usa'],
 		'bg_bg' => ['Bulgarian_Bulgaria.1251'],
@@ -163,7 +166,7 @@ function zbx_locale_variants_win($language) {
 		'ka_ge' => ['Georgian_Georgia.65001', 'georgian'],
 		'de_de' => ['German_Germany.1252', 'german'],
 		'el_gr' => ['Greek_Greece.1253', 'greek'],
-		'he_IL' => ['Hebrew_Israel.1255', 'hebrew'],
+		'he_il' => ['Hebrew_Israel.1255', 'hebrew'],
 		'hu_hu' => ['Hungarian_Hungary.1250', 'hungarian'],
 		'id_id' => ['Indonesian_indonesia.1252', 'indonesian'],
 		'it_it' => ['Italian_Italy.1252', 'italian'],
@@ -171,6 +174,7 @@ function zbx_locale_variants_win($language) {
 		'ja_jp' => ['Japanese_Japan.932', 'japanese'],
 		'lv_lv' => ['Latvian_Latvia.1257', 'latvian'],
 		'lt_lt' => ['Lithuanian_Lithuania.1257', 'lithuanian'],
+		'no_no' => ['Norwegian_Norway.1252', 'norwegian'],
 		'fa_ir' => ['Farsi_Iran.1256', 'farsi'],
 		'pl_pl' => ['Polish_Poland.1250', 'polish'],
 		'pt_br' => ['Portuguese_Brazil.1252', 'portuguese-brazil'],
@@ -184,5 +188,6 @@ function zbx_locale_variants_win($language) {
 		'uk_ua' => ['Ukrainian_Ukraine.1251', 'ukrainian'],
 		'vi_vn' => ['Vietnamese_Viet Nam.1258', 'vietnamese']
 	];
-	return $winLanguageName[strtolower($language)];
+
+	return $win_language_names[strtolower($language)];
 }
