@@ -124,13 +124,20 @@ function getActionsBySysmap($sysmap, array $options = []) {
 	]);
 
 	$triggers = API::Trigger()->get([
-		'output' => ['triggerid'],
-		'selectHosts' => ['hostid', 'status'],
+		'output' => [],
 		'triggerids' => $triggerIds,
 		'preservekeys' => true,
 		'nopermissions' => true
 	]);
 
+	$moniored_triggers = API::Trigger()->get([
+		'output' => [],
+		'triggerids' => array_keys($triggers),
+		'monitored' => true,
+		'nopermissions' => true,
+		'preservekeys' => true
+	]);
+
 	$host_groups = API::HostGroup()->get([
 		'output' => ['groupid'],
 		'groupids' => $host_groupids,
@@ -187,14 +194,8 @@ function getActionsBySysmap($sysmap, array $options = []) {
 				$gotos['showEvents'] = false;
 				foreach ($elem['elements'] as $element) {
 					if (array_key_exists($element['triggerid'], $triggers)) {
-						$trigger = $triggers[$element['triggerid']];
-
-						foreach ($trigger['hosts'] as $host) {
-							if ($host['status'] == HOST_STATUS_MONITORED) {
-								$gotos['showEvents'] = true;
-
-								break;
-							}
+						if (array_key_exists($element['triggerid'], $moniored_triggers)) {
+							$gotos['showEvents'] = true;
 						}
 
 						$gotos['events']['triggerids'][] = $element['triggerid'];
@@ -417,6 +418,7 @@ function getTriggersInfo($selement, $i, $showUnack) {
 		'info' => [],
 		'iconid' => $selement['iconid_off']
 	];
+	$config = select_config();
 
 	if ($i['problem'] && ($i['problem_unack'] && $showUnack == EXTACK_OPTION_UNACK
 			|| in_array($showUnack, [EXTACK_OPTION_ALL, EXTACK_OPTION_BOTH]))) {
@@ -444,7 +446,7 @@ function getTriggersInfo($selement, $i, $showUnack) {
 
 		$info['info']['unack'] = [
 			'msg' => $msg,
-			'color' => ($i['priority'] > 3) ? 'FF0000' : '960000'
+			'color' => $i['problem_unack'] ? $config['problem_unack_color'] : $config['problem_ack_color']
 		];
 
 		if (!array_key_exists('maintenance_title', $i)) {
@@ -476,7 +478,7 @@ function getTriggersInfo($selement, $i, $showUnack) {
 		$info['icon_type'] = SYSMAP_ELEMENT_ICON_OFF;
 		$info['info']['ok'] = [
 			'msg' => _('OK'),
-			'color' => '009600'
+			'color' => $i['ack'] ? $config['ok_ack_color'] : $config['ok_unack_color']
 		];
 	}
 
@@ -501,6 +503,7 @@ function getHostsInfo($selement, $i, $show_unack) {
 		'iconid' => $selement['iconid_off']
 	];
 	$hasProblem = false;
+	$config = select_config();
 
 	if ($i['problem']) {
 		if (in_array($show_unack, [EXTACK_OPTION_ALL, EXTACK_OPTION_BOTH])) {
@@ -528,14 +531,14 @@ function getHostsInfo($selement, $i, $show_unack) {
 
 			$info['info']['problem'] = [
 				'msg' => $msg,
-				'color' => ($i['priority'] > 3) ? 'FF0000' : '960000'
+				'color' => $i['problem_unack'] ? $config['problem_unack_color'] : $config['problem_ack_color']
 			];
 		}
 
 		if (in_array($show_unack, [EXTACK_OPTION_UNACK, EXTACK_OPTION_BOTH]) && $i['problem_unack']) {
 			$info['info']['unack'] = [
 				'msg' => $i['problem_unack'].' '._('Unacknowledged'),
-				'color' => '960000'
+				'color' => $config['problem_unack_color']
 			];
 		}
 
@@ -568,7 +571,7 @@ function getHostsInfo($selement, $i, $show_unack) {
 		$info['icon_type'] = SYSMAP_ELEMENT_ICON_OFF;
 		$info['info']['ok'] = [
 			'msg' => _('OK'),
-			'color' => '009600'
+			'color' => $i['ack'] ? $config['ok_ack_color'] : $config['ok_unack_color']
 		];
 	}
 
@@ -594,6 +597,7 @@ function getHostGroupsInfo($selement, $i, $show_unack) {
 	];
 	$hasProblem = false;
 	$hasStatus = false;
+	$config = select_config();
 
 	if ($i['problem']) {
 		if (in_array($show_unack, [EXTACK_OPTION_ALL, EXTACK_OPTION_BOTH])) {
@@ -621,14 +625,14 @@ function getHostGroupsInfo($selement, $i, $show_unack) {
 
 			$info['info']['problem'] = [
 				'msg' => $msg,
-				'color' => ($i['priority'] > 3) ? 'FF0000' : '960000'
+				'color' => $i['problem_unack'] ? $config['problem_unack_color'] : $config['problem_ack_color']
 			];
 		}
 
 		if (in_array($show_unack, [EXTACK_OPTION_UNACK, EXTACK_OPTION_BOTH]) && $i['problem_unack']) {
 			$info['info']['unack'] = [
 				'msg' => $i['problem_unack'].' '._('Unacknowledged'),
-				'color' => '960000'
+				'color' => $config['problem_unack_color']
 			];
 		}
 
@@ -668,7 +672,7 @@ function getHostGroupsInfo($selement, $i, $show_unack) {
 		$info['iconid'] = $selement['iconid_off'];
 		$info['info']['ok'] = [
 			'msg' => _('OK'),
-			'color' => '009600'
+			'color' => $i['ack'] ? $config['ok_ack_color'] : $config['ok_unack_color']
 		];
 	}
 
@@ -695,6 +699,7 @@ function getMapsInfo($selement, $i, $show_unack) {
 
 	$hasProblem = false;
 	$hasStatus = false;
+	$config = select_config();
 
 	if ($i['problem']) {
 		if (in_array($show_unack, [EXTACK_OPTION_ALL, EXTACK_OPTION_BOTH])) {
@@ -722,14 +727,14 @@ function getMapsInfo($selement, $i, $show_unack) {
 
 			$info['info']['problem'] = [
 				'msg' => $msg,
-				'color' => ($i['priority'] > 3) ? 'FF0000' : '960000'
+				'color' => $i['problem_unack'] ? $config['problem_unack_color'] : $config['problem_ack_color']
 			];
 		}
 
 		if (in_array($show_unack, [EXTACK_OPTION_UNACK, EXTACK_OPTION_BOTH]) && $i['problem_unack']) {
 			$info['info']['unack'] = [
 				'msg' => $i['problem_unack'].' '._('Unacknowledged'),
-				'color' => '960000'
+				'color' => $config['problem_unack_color']
 			];
 		}
 
@@ -768,7 +773,7 @@ function getMapsInfo($selement, $i, $show_unack) {
 		$info['iconid'] = $selement['iconid_off'];
 		$info['info']['ok'] = [
 			'msg' => _('OK'),
-			'color' => '009600'
+			'color' => $i['ack'] ? $config['ok_ack_color'] : $config['ok_unack_color']
 		];
 	}
 
@@ -968,14 +973,28 @@ function getSelementsInfo($sysmap, array $options = []) {
 			'selectLastEvent' => ['acknowledged'],
 			'triggerids' => array_keys($triggerIdToSelementIds),
 			'filter' => ['state' => null],
-			'nopermissions' => true
+			'nopermissions' => true,
+			'preservekeys' => true
+		]);
+
+		$monitored_triggers = API::Trigger()->get([
+			'output' => [],
+			'triggerids' => array_keys($triggers),
+			'monitored' => true,
+			'nopermissions' => true,
+			'preservekeys' => true
 		]);
 
 		foreach ($triggers as $trigger) {
+			if (!array_key_exists($trigger['triggerid'], $monitored_triggers)) {
+				$trigger['status'] = TRIGGER_STATUS_DISABLED;
+			}
+
 			foreach ($triggerIdToSelementIds[$trigger['triggerid']] as $belongs_to_sel) {
 				$selements[$belongs_to_sel]['triggers'][$trigger['triggerid']] = $trigger;
 			}
 		}
+		unset($triggers, $monitored_triggers);
 	}
 
 	// triggers from submaps, skip dependent
@@ -991,11 +1010,24 @@ function getSelementsInfo($sysmap, array $options = []) {
 			'only_true' => true
 		]);
 
+		$monitored_triggers = API::Trigger()->get([
+			'output' => [],
+			'triggerids' => array_keys($triggers),
+			'monitored' => true,
+			'nopermissions' => true,
+			'preservekeys' => true
+		]);
+
 		foreach ($triggers as $trigger) {
+			if (!array_key_exists($trigger['triggerid'], $monitored_triggers)) {
+				$trigger['status'] = TRIGGER_STATUS_DISABLED;
+			}
+
 			foreach ($subSysmapTriggerIdToSelementIds[$trigger['triggerid']] as $belongs_to_sel) {
 				$selements[$belongs_to_sel]['triggers'][$trigger['triggerid']] = $trigger;
 			}
 		}
+		unset($triggers, $monitored_triggers);
 	}
 
 	$monitored_hostids = [];
