@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -310,7 +310,7 @@ function createServiceMonitoringTree(array $services, array $slaData, $period, &
 }
 
 /**
- * Calculates the current IT service status based on it's child services.
+ * Calculates the current service status based on it's child services.
  *
  * The new statuses are written to the $services array in the "newStatus" property.
  *
@@ -380,7 +380,7 @@ function calculateItServiceStatusByTrigger($triggerStatus, $triggerValue, $trigg
 }
 
 /**
- * Updates the status of all IT services
+ * Updates the status of all services
  */
 function updateItServices() {
 	$servicesLinks = [];
@@ -509,3 +509,36 @@ function checkServiceTime(array $serviceTime) {
 		throw new APIException(ZBX_API_ERROR_PARAMETERS, _('Service start time must be less than end time.'));
 	}
 }
+
+/**
+ * Method to sort list of Services by 'sortorder' field and then by 'name' field if more entries has same 'sortorder'
+ * value. Separate method is needed because entries make multilevel hierarchy and branches also must be sorted according
+ * fields 'sortorder' and 'name'.
+ *
+ * @param array $services
+ *
+ * @return void
+ */
+function sortServices(array &$services) {
+	$sort_options = [
+		['field' => 'sortorder', 'order' => ZBX_SORT_UP],
+		['field' => 'name', 'order' => ZBX_SORT_UP]
+	];
+
+	// Sort first level entries.
+	CArrayHelper::sort($services, $sort_options);
+
+	// Sort dependencies.
+	foreach ($services as &$service) {
+		if ($service['dependencies']) {
+			foreach ($service['dependencies'] as &$dependent_item) {
+				$dependent_item['name'] = $services[$dependent_item['serviceid']]['name'];
+				$dependent_item['sortorder'] = $services[$dependent_item['serviceid']]['sortorder'];
+			}
+			unset($dependent_item);
+
+			CArrayHelper::sort($service['dependencies'], $sort_options);
+		}
+	}
+	unset($service);
+}
