@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,23 +24,22 @@ if (isset($_REQUEST['screenitemid'])) {
 	$action .= '&screenitemid='.getRequest('screenitemid');
 }
 
-// create screen form
-$screenForm = (new CForm('post', $action))->setName('screen_item_form');
+$form = (new CForm('post', $action))
+	->setName('screen_item_form')
+	->addVar('screenid', getRequest('screenid'));
 
 if ($data['screen']['templateid'] != 0) {
-	$screenForm->addVar('templateid', $data['screen']['templateid']);
+	$form->addVar('templateid', $data['screen']['templateid']);
 }
 
-// create screen form list
-$screenFormList = (new CFormList())->addVar('screenid', $_REQUEST['screenid']);
-
-if (isset($_REQUEST['screenitemid'])) {
-	$screenFormList->addVar('screenitemid', $_REQUEST['screenitemid']);
+if (hasRequest('screenitemid')) {
+	$form->addVar('screenitemid', getRequest('screenitemid'));
 	$screenItems = zbx_toHash($this->data['screen']['screenitems'], 'screenitemid');
 }
 else {
-	$screenFormList->addVar('x', $_REQUEST['x']);
-	$screenFormList->addVar('y', $_REQUEST['y']);
+	$form
+		->addVar('x', getRequest('x'))
+		->addVar('y', getRequest('y'));
 }
 
 if (isset($_REQUEST['screenitemid']) && !isset($_REQUEST['form_refresh'])) {
@@ -84,14 +83,16 @@ $screenResources = screen_resources();
 if ($this->data['screen']['templateid']) {
 	unset(
 		$screenResources[SCREEN_RESOURCE_DATA_OVERVIEW], $screenResources[SCREEN_RESOURCE_ACTIONS],
-		$screenResources[SCREEN_RESOURCE_EVENTS], $screenResources[SCREEN_RESOURCE_HOSTS_INFO],
+		$screenResources[SCREEN_RESOURCE_EVENTS], $screenResources[SCREEN_RESOURCE_HOST_INFO],
 		$screenResources[SCREEN_RESOURCE_MAP], $screenResources[SCREEN_RESOURCE_SCREEN],
 		$screenResources[SCREEN_RESOURCE_SERVER_INFO], $screenResources[SCREEN_RESOURCE_HOSTGROUP_TRIGGERS],
 		$screenResources[SCREEN_RESOURCE_HOST_TRIGGERS], $screenResources[SCREEN_RESOURCE_SYSTEM_STATUS],
-		$screenResources[SCREEN_RESOURCE_TRIGGERS_INFO], $screenResources[SCREEN_RESOURCE_TRIGGERS_OVERVIEW]
+		$screenResources[SCREEN_RESOURCE_TRIGGER_INFO], $screenResources[SCREEN_RESOURCE_TRIGGER_OVERVIEW]
 	);
 }
-$screenFormList->addRow(_('Resource'), new CComboBox('resourcetype', $resourceType, 'submit()', $screenResources));
+
+$screenFormList = (new CFormList())
+	->addRow(_('Resource'), new CComboBox('resourcetype', $resourceType, 'submit()', $screenResources));
 
 /*
  * Screen item: Graph
@@ -119,7 +120,7 @@ if ($resourceType == SCREEN_RESOURCE_GRAPH) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=graphs&srcfld1=graphid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].'");'
 			);
 	}
@@ -127,11 +128,11 @@ if ($resourceType == SCREEN_RESOURCE_GRAPH) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=graphs&srcfld1=graphid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&real_hosts=1&with_graphs=1");');
 	}
 
-	$screenFormList->addVar('resourceid', $id);
+	$form->addVar('resourceid', $id);
 	$screenFormList->addRow(_('Graph'), [
 		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -166,7 +167,7 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_GRAPH) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=graph_prototypes&srcfld1=graphid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&only_hostid='.$this->data['screen']['templateid'].
 				'&templated_hosts=1");');
 	}
@@ -174,11 +175,11 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_GRAPH) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=graph_prototypes&srcfld1=graphid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&real_hosts=1");');
 	}
 
-	$screenFormList->addVar('resourceid', $id);
+	$form->addVar('resourceid', $id);
 	$screenFormList->addRow(_('Graph prototype'), [
 		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -218,7 +219,7 @@ elseif ($resourceType == SCREEN_RESOURCE_SIMPLE_GRAPH) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].
 				'&numeric=1");');
 	}
@@ -226,11 +227,11 @@ elseif ($resourceType == SCREEN_RESOURCE_SIMPLE_GRAPH) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&real_hosts=1&with_simple_graph_items=1&numeric=1");');
 	}
 
-	$screenFormList->addVar('resourceid', $id);
+	$form->addVar('resourceid', $id);
 	$screenFormList->addRow(_('Item'), [
 		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -265,7 +266,7 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_SIMPLE_GRAPH) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=item_prototypes&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].
 				'&numeric=1");');
 	}
@@ -273,11 +274,11 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_SIMPLE_GRAPH) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=item_prototypes&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&real_hosts=1&with_discovery_rule=1&items=1&numeric=1");');
 	}
 
-	$screenFormList->addVar('resourceid', $id);
+	$form->addVar('resourceid', $id);
 	$screenFormList->addRow(_('Item prototype'), [
 		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -307,14 +308,14 @@ elseif ($resourceType == SCREEN_RESOURCE_MAP) {
 		$caption = $map['name'];
 	}
 
-	$screenFormList->addVar('resourceid', $id);
+	$form->addVar('resourceid', $id);
 	$screenFormList->addRow(_('Map'), [
 		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 		(new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=sysmaps&srcfld1=sysmapid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption");'
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption");'
 		)
 	]);
 }
@@ -327,9 +328,10 @@ elseif ($resourceType == SCREEN_RESOURCE_PLAIN_TEXT) {
 	$id = 0;
 
 	$items = API::Item()->get([
-		'itemids' => $resourceId,
+		'output' => ['itemid', 'hostid', 'key_', 'name'],
 		'selectHosts' => ['name'],
-		'output' => ['itemid', 'hostid', 'key_', 'name']
+		'itemids' => $resourceId,
+		'webitems' => true
 	]);
 
 	if ($items) {
@@ -341,11 +343,13 @@ elseif ($resourceType == SCREEN_RESOURCE_PLAIN_TEXT) {
 		$caption = $item['host']['name'].NAME_DELIMITER.$item['name_expanded'];
 	}
 
+	$form->addVar('resourceid', $id);
+
 	if ($this->data['screen']['templateid']) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&templated_hosts=1&only_hostid='.$this->data['screen']['templateid'].'");'
 			);
 	}
@@ -353,12 +357,11 @@ elseif ($resourceType == SCREEN_RESOURCE_PLAIN_TEXT) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=items&srcfld1=itemid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption&real_hosts=1");'
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption&real_hosts=1");'
 			);
 	}
 
 	$screenFormList
-		->addVar('resourceid', $id)
 		->addRow(_('Item'), [
 			(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -369,19 +372,16 @@ elseif ($resourceType == SCREEN_RESOURCE_PLAIN_TEXT) {
 		)
 		->addRow(_('Show text as HTML'), (new CCheckBox('style'))->setChecked($style == 1));
 }
-
-/*
- * Screen item: Status of triggers
- */
 elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESOURCE_HOST_TRIGGERS])) {
+	// Screen item: Triggers
+
 	$data = [];
 
 	if ($resourceType == SCREEN_RESOURCE_HOSTGROUP_TRIGGERS) {
 		if ($resourceId > 0) {
 			$data = API::HostGroup()->get([
 				'groupids' => $resourceId,
-				'output' => ['groupid', 'name'],
-				'editable' => true
+				'output' => ['groupid', 'name']
 			]);
 
 			if ($data) {
@@ -393,12 +393,11 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESO
 			(new CMultiSelect([
 				'name' => 'resourceid',
 				'objectName' => 'hostGroup',
-				'objectOptions' => ['editable' => true],
 				'data' => $data ? [['id' => $data['groupid'], 'name' => $data['name']]] : null,
 				'defaultValue' => 0,
 				'selectedLimit' => 1,
 				'popup' => [
-					'parameters' => 'srctbl=host_groups&dstfrm='.$screenForm->getName().'&dstfld1=resourceid'.
+					'parameters' => 'srctbl=host_groups&dstfrm='.$form->getName().'&dstfld1=resourceid'.
 						'&srcfld1=groupid'
 				]
 			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
@@ -408,8 +407,7 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESO
 		if ($resourceId > 0) {
 			$data = API::Host()->get([
 				'hostids' => $resourceId,
-				'output' => ['hostid', 'name'],
-				'editable' => true
+				'output' => ['hostid', 'name']
 			]);
 
 			if ($data) {
@@ -421,12 +419,11 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_HOSTGROUP_TRIGGERS, SCREEN_RESO
 			(new CMultiSelect([
 				'name' => 'resourceid',
 				'objectName' => 'hosts',
-				'objectOptions' => ['editable' => true],
 				'data' => $data ? [['id' => $data['hostid'], 'name' => $data['name']]] : null,
 				'defaultValue' => 0,
 				'selectedLimit' => 1,
 				'popup' => [
-					'parameters' => 'srctbl=hosts&dstfrm='.$screenForm->getName().'&dstfld1=resourceid'.
+					'parameters' => 'srctbl=hosts&dstfrm='.$form->getName().'&dstfld1=resourceid'.
 						'&srcfld1=hostid'
 				]
 			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
@@ -475,20 +472,19 @@ elseif ($resourceType == SCREEN_RESOURCE_EVENTS) {
 	$screenFormList->addRow(_('Show lines'),
 		(new CNumericBox('elements', $elements, 3))->setWidth(ZBX_TEXTAREA_NUMERIC_STANDARD_WIDTH)
 	);
-	$screenFormList->addVar('resourceid', 0);
+	$form->addVar('resourceid', 0);
 }
 
 /*
  * Screen item: Overviews
  */
-elseif (in_array($resourceType, [SCREEN_RESOURCE_TRIGGERS_OVERVIEW, SCREEN_RESOURCE_DATA_OVERVIEW])) {
+elseif (in_array($resourceType, [SCREEN_RESOURCE_TRIGGER_OVERVIEW, SCREEN_RESOURCE_DATA_OVERVIEW])) {
 	$data = [];
 
 	if ($resourceId > 0) {
 		$data = API::HostGroup()->get([
 			'groupids' => $resourceId,
-			'output' => ['groupid', 'name'],
-			'editable' => true
+			'output' => ['groupid', 'name']
 		]);
 
 		if ($data) {
@@ -500,11 +496,10 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_TRIGGERS_OVERVIEW, SCREEN_RESOU
 		(new CMultiSelect([
 			'name' => 'resourceid',
 			'objectName' => 'hostGroup',
-			'objectOptions' => ['editable' => true],
 			'data' => $data ? [['id' => $data['groupid'], 'name' => $data['name']]] : null,
 			'selectedLimit' => 1,
 			'popup' => [
-				'parameters' => 'srctbl=host_groups&dstfrm='.$screenForm->getName().'&dstfld1=resourceid'.
+				'parameters' => 'srctbl=host_groups&dstfrm='.$form->getName().'&dstfld1=resourceid'.
 					'&srcfld1=groupid'
 			]
 		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
@@ -541,29 +536,26 @@ elseif ($resourceType == SCREEN_RESOURCE_SCREEN) {
 		}
 	}
 
-	$screenFormList->addVar('resourceid', $id);
+	$form->addVar('resourceid', $id);
 	$screenFormList->addRow(_('Screen'), [
 		(new CTextBox('caption', $caption, true))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 		(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 		(new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('javascript: return PopUp("popup.php?srctbl=screens2&srcfld1=screenid&srcfld2=name'.
-				'&dstfrm='.$screenForm->getName().'&dstfld1=resourceid&dstfld2=caption'.
+				'&dstfrm='.$form->getName().'&dstfld1=resourceid&dstfld2=caption'.
 				'&screenid='.$_REQUEST['screenid'].'");')
 	]);
 }
+elseif ($resourceType == SCREEN_RESOURCE_HOST_INFO || $resourceType == SCREEN_RESOURCE_TRIGGER_INFO) {
+	// Screen item: Host info
 
-/*
- * Screen item: Hosts info
- */
-elseif ($resourceType == SCREEN_RESOURCE_HOSTS_INFO || $resourceType == SCREEN_RESOURCE_TRIGGERS_INFO) {
 	$data = [];
 
 	if ($resourceId > 0) {
 		$data = API::HostGroup()->get([
 			'groupids' => $resourceId,
-			'output' => ['groupid', 'name'],
-			'editable' => true
+			'output' => ['groupid', 'name']
 		]);
 
 		if ($data) {
@@ -575,12 +567,11 @@ elseif ($resourceType == SCREEN_RESOURCE_HOSTS_INFO || $resourceType == SCREEN_R
 		(new CMultiSelect([
 			'name' => 'resourceid',
 			'objectName' => 'hostGroup',
-			'objectOptions' => ['editable' => true],
 			'data' => $data ? [['id' => $data['groupid'], 'name' => $data['name']]] : null,
 			'defaultValue' => 0,
 			'selectedLimit' => 1,
 			'popup' => [
-				'parameters' => 'srctbl=host_groups&dstfrm='.$screenForm->getName().'&dstfld1=resourceid'.
+				'parameters' => 'srctbl=host_groups&dstfrm='.$form->getName().'&dstfld1=resourceid'.
 					'&srcfld1=groupid'
 			]
 		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
@@ -617,19 +608,19 @@ elseif ($resourceType == SCREEN_RESOURCE_CLOCK) {
 	]));
 
 	if (TIME_TYPE_HOST == $style) {
-		$screenFormList->addVar('resourceid', $resourceId);
+		$form->addVar('resourceid', $resourceId);
 
 		if ($this->data['screen']['templateid']) {
 			$selectButton = (new CButton('select', _('Select')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick("javascript: return PopUp('popup.php?dstfrm=".$screenForm->getName().
+				->onClick("javascript: return PopUp('popup.php?dstfrm=".$form->getName().
 					'&dstfld1=resourceid&dstfld2=caption&srctbl=items&srcfld1=itemid&srcfld2=name&templated_hosts=1'.
 					'&only_hostid='.$this->data['screen']['templateid']."');");
 		}
 		else {
 			$selectButton = (new CButton('select', _('Select')))
 				->addClass(ZBX_STYLE_BTN_GREY)
-				->onClick("javascript: return PopUp('popup.php?dstfrm=".$screenForm->getName().'&dstfld1=resourceid'.
+				->onClick("javascript: return PopUp('popup.php?dstfrm=".$form->getName().'&dstfld1=resourceid'.
 					"&dstfld2=caption&srctbl=items&srcfld1=itemid&srcfld2=name&real_hosts=1');");
 		}
 		$screenFormList->addRow(_('Item'), [
@@ -639,14 +630,14 @@ elseif ($resourceType == SCREEN_RESOURCE_CLOCK) {
 		]);
 	}
 	else {
-		$screenFormList->addVar('caption', $caption);
+		$form->addVar('caption', $caption);
 	}
 }
 
 /*
  * Append common fields
  */
-if (in_array($resourceType, [SCREEN_RESOURCE_HOSTS_INFO, SCREEN_RESOURCE_TRIGGERS_INFO])) {
+if (in_array($resourceType, [SCREEN_RESOURCE_HOST_INFO, SCREEN_RESOURCE_TRIGGER_INFO])) {
 	$screenFormList->addRow(_('Style'),
 		(new CRadioButtonList('style', (int) $style))
 			->addValue(_('Horizontal'), STYLE_HORIZONTAL)
@@ -654,7 +645,7 @@ if (in_array($resourceType, [SCREEN_RESOURCE_HOSTS_INFO, SCREEN_RESOURCE_TRIGGER
 			->setModern(true)
 	);
 }
-elseif (in_array($resourceType, [SCREEN_RESOURCE_TRIGGERS_OVERVIEW, SCREEN_RESOURCE_DATA_OVERVIEW])) {
+elseif (in_array($resourceType, [SCREEN_RESOURCE_TRIGGER_OVERVIEW, SCREEN_RESOURCE_DATA_OVERVIEW])) {
 	$screenFormList->addRow(_('Hosts location'),
 		(new CRadioButtonList('style', (int) $style))
 			->addValue(_('Left'), STYLE_LEFT)
@@ -663,14 +654,14 @@ elseif (in_array($resourceType, [SCREEN_RESOURCE_TRIGGERS_OVERVIEW, SCREEN_RESOU
 	);
 }
 elseif ($resourceType != SCREEN_RESOURCE_CLOCK) {
-	$screenFormList->addVar('style', 0);
+	$form->addVar('style', 0);
 }
 
 if (in_array($resourceType, [SCREEN_RESOURCE_URL])) {
 	$screenFormList->addRow(_('URL'), (new CTextBox('url', $url))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH));
 }
 else {
-	$screenFormList->addVar('url', '');
+	$form->addVar('url', '');
 }
 
 $resourcesWithWidthAndHeight = [
@@ -690,8 +681,9 @@ if (in_array($resourceType, $resourcesWithWidthAndHeight)) {
 	);
 }
 else {
-	$screenFormList->addVar('width', 500);
-	$screenFormList->addVar('height', 100);
+	$form
+		->addVar('width', 500)
+		->addVar('height', 100);
 }
 
 $resourcesWithHAlign = [
@@ -713,7 +705,7 @@ if (in_array($resourceType, $resourcesWithHAlign)) {
 	);
 }
 else {
-	$screenFormList->addVar('halign', 0);
+	$form->addVar('halign', 0);
 }
 
 $screenFormList->addRow(_('Vertical align'),
@@ -743,14 +735,12 @@ if ($this->data['screen']['templateid'] == 0 && in_array($resourceType, $resourc
 	$screenFormList->addRow(_('Dynamic item'), (new CCheckBox('dynamic'))->setChecked($dynamic == 1));
 }
 
-// append tabs to form
-$screenTab = new CTabView();
-$screenTab->setAttribute('style', 'text-align: left;');
-$screenTab->addTab('screenTab', _('Screen cell configuration'), $screenFormList);
+// append list to form
+$form->addItem($screenFormList);
 
 // append buttons to form
 if (isset($_REQUEST['screenitemid'])) {
-	$screenTab->setFooter(makeFormFooter(
+	$form->addItem(makeFormFooter(
 		new CSubmit('update', _('Update')),
 		[
 			new CButtonDelete(null, url_params(['form', 'screenid', 'templateid', 'screenitemid'])),
@@ -759,12 +749,10 @@ if (isset($_REQUEST['screenitemid'])) {
 	));
 }
 else {
-	$screenTab->setFooter(makeFormFooter(
+	$form->addItem(makeFormFooter(
 		new CSubmit('add', _('Add')),
 		[new CButtonCancel(url_params(['screenid', 'templateid']))]
 	));
 }
 
-$screenForm->addItem($screenTab);
-
-return $screenForm;
+return $form;
