@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ $queueRequests = [
 	QUEUE_OVERVIEW_BY_PROXY => CZabbixServer::QUEUE_OVERVIEW_BY_PROXY,
 	QUEUE_DETAILS => CZabbixServer::QUEUE_DETAILS
 ];
-$queueData = $zabbixServer->getQueue($queueRequests[$config], get_cookie('zbx_sessionid'));
+$queueData = $zabbixServer->getQueue($queueRequests[$config], get_cookie('zbx_sessionid'), QUEUE_DETAIL_ITEM_COUNT);
 
 // check for errors error
 if ($zabbixServer->getError()) {
@@ -67,11 +67,11 @@ $widget = (new CWidget())
 	->setControls((new CForm('get'))
 		->cleanItems()
 		->addItem((new CList())
-			->addItem((new CComboBox('config', $config, 'submit();'))
-				->addItem(QUEUE_OVERVIEW, _('Overview'))
-				->addItem(QUEUE_OVERVIEW_BY_PROXY, _('Overview by proxy'))
-				->addItem(QUEUE_DETAILS, _('Details'))
-			)
+			->addItem((new CComboBox('config', $config, 'submit();', [
+				QUEUE_OVERVIEW => _('Overview'),
+				QUEUE_OVERVIEW_BY_PROXY => _('Overview by proxy'),
+				QUEUE_DETAILS => _('Details')
+			])))
 		)
 	);
 
@@ -255,7 +255,7 @@ elseif ($config == QUEUE_DETAILS) {
 			continue;
 		}
 
-		// display only the first 500 items
+		// display only the first QUEUE_DETAIL_ITEM_COUNT items (can only occur when using old server versions)
 		$i++;
 		if ($i > QUEUE_DETAIL_ITEM_COUNT) {
 			break;
@@ -267,7 +267,7 @@ elseif ($config == QUEUE_DETAILS) {
 		$table->addRow([
 			zbx_date2str(DATE_TIME_FORMAT_SECONDS, $itemData['nextcheck']),
 			zbx_date2age($itemData['nextcheck']),
-			(isset($proxies[$hosts[$item['hostid']]['proxy_hostid']]))
+			isset($proxies[$hosts[$item['hostid']]['proxy_hostid']])
 				? $proxies[$hosts[$item['hostid']]['proxy_hostid']]['host'].NAME_DELIMITER.$host['name']
 				: $host['name'],
 			$item['name_expanded']
@@ -277,20 +277,30 @@ elseif ($config == QUEUE_DETAILS) {
 
 // display the table footer
 if ($config == QUEUE_OVERVIEW_BY_PROXY) {
-	$table->setFooter(
-		new CCol(_('Total').': '.$table->getNumRows())
-	);
+	$total = _('Total').': '.$table->getNumRows();
 }
 elseif ($config == QUEUE_DETAILS) {
-	$table->setFooter(
-		new CCol(_('Total').': '.$table->getNumRows().
-		(count($queueData) > QUEUE_DETAIL_ITEM_COUNT ? ' ('._('Truncated').')' : ''))
-	);
+	$total = _s('Displaying %1$s of %2$s found', $table->getNumRows(), $zabbixServer->getTotalCount());
+}
+else {
+	$total = null;
+}
+
+if ($total !== null) {
+	$total = (new CDiv())
+		->addClass(ZBX_STYLE_TABLE_PAGING)
+		->addItem((new CDiv())
+			->addClass(ZBX_STYLE_PAGING_BTN_CONTAINER)
+			->addItem((new CDiv())
+				->addClass(ZBX_STYLE_TABLE_STATS)
+				->addItem($total)
+			)
+		);
 }
 
 $widget
 	->addItem($table)
+	->addItem($total)
 	->show();
 
-
 require_once dirname(__FILE__).'/include/page_footer.php';
