@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -45,7 +45,9 @@ require_once dirname(__FILE__).'/include/page_header.php';
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
 	'screens' =>		[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,			null],
-	'screenid' =>		[T_ZBX_INT, O_NO,	P_SYS,	DB_ID,			'isset({form}) && {form} == "update"'],
+	'screenid' =>		[T_ZBX_INT, O_NO,  P_SYS,	DB_ID,
+		'isset({form}) && ({form} === "update" || {form} === "full_clone")'
+	],
 	'templateid' =>		[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,			null],
 	'name' =>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,		'isset({add}) || isset({update})', _('Name')],
 	'hsize' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(SCREEN_MIN_SIZE, SCREEN_MAX_SIZE),
@@ -205,6 +207,36 @@ if (hasRequest('add') || hasRequest('update')) {
 		$messageSuccess = _('Screen added');
 		$messageFailed = _('Cannot add screen');
 
+		if (getRequest('form') === 'full_clone') {
+			$output = ['resourcetype', 'resourceid', 'width', 'height', 'x', 'y', 'colspan', 'rowspan', 'elements',
+				'valign', 'haligh', 'style', 'url', 'max_columns'
+			];
+
+			if (hasRequest('templateid')) {
+				$screen['screenitems'] = API::TemplateScreenItem()->get([
+					'output' => $output,
+					'screenids' => [getRequest('screenid')]
+				]);
+			}
+			else {
+				array_push($output, 'dynamic', 'sort_triggers', 'application');
+
+				$screen['screenitems'] = API::ScreenItem()->get([
+					'output' => $output,
+					'screenids' => [getRequest('screenid')]
+				]);
+			}
+
+			$max_x = $screen['hsize'] - 1;
+			$max_y = $screen['vsize'] - 1;
+
+			foreach ($screen['screenitems'] as $key => $screen_item) {
+				if ($screen_item['x'] > $max_x || $screen_item['y'] > $max_y) {
+					unset($screen['screenitems'][$key]);
+				}
+			}
+		}
+
 		if (hasRequest('templateid')) {
 			$screen['templateid'] = getRequest('templateid');
 
@@ -388,7 +420,7 @@ else {
 			'output' => ['screenid', 'name', 'hsize', 'vsize'],
 			'templateids' => $data['templateid'],
 			'sortfield' => $sortField,
-			'limit' => $config['search_limit'],
+			'limit' => $config['search_limit'] + 1,
 			'editable' => true,
 			'preservekeys' => true
 		]);
@@ -410,7 +442,7 @@ else {
 		$data['screens'] = API::Screen()->get([
 			'output' => ['screenid', 'name', 'hsize', 'vsize'],
 			'sortfield' => $sortField,
-			'limit' => $config['search_limit'],
+			'limit' => $config['search_limit'] + 1,
 			'search' => [
 				'name' => ($data['filter']['name'] === '') ? null : $data['filter']['name']
 			],
