@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 
 require_once dirname(__FILE__).'/include/config.inc.php';
+require_once dirname(__FILE__).'/include/hostgroups.inc.php';
 
 $page['file'] = 'tr_status.php';
 $page['title'] = _('Triggers');
@@ -61,10 +62,10 @@ check_fields($fields);
 /*
  * Permissions
  */
-if (getRequest('groupid') && !API::HostGroup()->isReadable([getRequest('groupid')])) {
+if (getRequest('groupid') && !isReadableHostGroups([getRequest('groupid')])) {
 	access_deny();
 }
-if (getRequest('hostid') && !API::Host()->isReadable([getRequest('hostid')])) {
+if (getRequest('hostid') && !isReadableHosts([getRequest('hostid')])) {
 	access_deny();
 }
 
@@ -377,7 +378,7 @@ if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
 		'object' => EVENT_OBJECT_TRIGGER,
 		'value' => TRIGGER_VALUE_TRUE,
 		'objectids' => zbx_objectValues($triggers, 'triggerid'),
-		'time_from' => time() - $config['event_expire'] * SEC_PER_DAY,
+		'time_from' => time() - timeUnitToSeconds($config['event_expire']),
 		'time_till' => time(),
 		'sortfield' => ['clock', 'eventid'],
 		'sortorder' => ZBX_SORT_DOWN
@@ -660,9 +661,7 @@ foreach ($triggers as $trigger) {
 	]);
 
 	if ($showEvents == EVENTS_OPTION_ALL || $showEvents == EVENTS_OPTION_NOT_ACK) {
-		$next_event_clock = time();
-
-		foreach (array_slice($trigger['events'], 0, $config['event_show_max']) as $enum => $event) {
+		foreach (array_slice($trigger['events'], 0, $config['event_show_max']) as $event) {
 			if ($showEvents == EVENTS_OPTION_NOT_ACK && $event['acknowledged']) {
 				continue;
 			}
@@ -705,10 +704,6 @@ foreach ($triggers as $trigger) {
 					'tr_events.php?triggerid='.$trigger['triggerid'].'&eventid='.$event['eventid']
 				);
 
-			if ($enum != 0) {
-				$next_event_clock = $trigger['events'][$enum - 1]['clock'];
-			}
-
 			$triggerTable->addRow(
 				(new CRow([
 					(new CCol())->setColSpan($config['event_ack_enable'] ? 3 : 2),
@@ -716,7 +711,7 @@ foreach ($triggers as $trigger) {
 					$clock,
 					$r_clock,
 					zbx_date2age($event['clock']),
-					zbx_date2age($next_event_clock, $event['clock']),
+					zbx_date2age($event['clock'], $event['r_clock']),
 					$config['event_ack_enable']
 						? ($event['value'] == TRIGGER_VALUE_TRUE)
 							? getEventAckState($event, $page['file'])
